//---------------------------------------------------------------------------
/*
   KNIHOVNA PRO VYPOCET KOEFICIENTU DIGITALNICH BIQUAD FILTRU
   (c) Ivo Strasil 2006
   Toto neni volne siritelny software.
 */




#include "filterdesign.h"

#include <math.h>

#define w0      (2*M_PI*(float)f0/(float)Fs)

#define Sign(x) (x>0.0 ? 1 : (x<0.0 ? -1 : 0))

//pomocne vypocty pro vypocet filtru
float dBToA (float dBGain) {            //odmocnina ze zesileni
	return pow(10,(dBGain/40));
}

float dBToK (float dBGain) {            //zesileni
	return pow(10,(dBGain/20));
}
unsigned char KToDb(float K) {  //vraci zesileni v dB+128
	return (unsigned char)(128+20.0*log10(K));
}


//prevod Q na Alpha
float QToAlpha(float Q, unsigned int f0) {
	return sin(w0)/(2*Q);
}
//prevody Q na jine veliciny
float BWToQ(float BW, unsigned int f0) {
	return 1/(2*sinh(log(2)/2*BW*w0/sin(w0)));
}
float SToQ(float S, unsigned int f0, float dbGain) {
	return 1/(sqrt((dBToA(dbGain) + 1/dBToA(dbGain))*(1/S - 1) + 2) );
}



//vlastni vypocet filtru
char ComputeFilter(filter * f, fdata * fd) {
	float A=dBToA(f->dbGain);
	float cs=cos(2*M_PI*f->f0/Fs);
	float alpha=sin(2*M_PI*f->f0/Fs)/(2*f->Q);
	switch (f->ftype) {
	case FTGAIN:
		fd->b0 =  A*A;
		fd->b1 =  0;
		fd->b2 =  0;
		fd->a0 =  1;
		fd->a1 =  0;
		fd->a2 =  0;
		break;
	case FTLPF:
		fd->b0 =  (1 - cs)/2;
		fd->b1 =   1 - cs;
		fd->b2 =  (1 - cs)/2;
		fd->a0 =   1 + alpha;
		fd->a1 =  -2*cs;
		fd->a2 =   1 - alpha;
		break;
	case FTHPF:
		fd->b0 =  (1 + cs)/2;
		fd->b1 = -(1 + cs);
		fd->b2 =  (1 + cs)/2;
		fd->a0 =   1 + alpha;
		fd->a1 =  -2*cs;
		fd->a2 =   1 - alpha;
		break;
	case FTBPF:
		fd->b0 =   alpha;
		fd->b1 =   0;
		fd->b2 =  -alpha;
		fd->a0 =   1 + alpha;
		fd->a1 =  -2*cs;
		fd->a2 =   1 - alpha;
		break;
	case FTNOTCH:
		fd->b0 =   1;
		fd->b1 =  -2*cs;
		fd->b2 =   1;
		fd->a0 =   1 + alpha;
		fd->a1 =  -2*cs ;
		fd->a2 =   1 - alpha;
		break;
	case FTAPF:
		fd->b0 =   1 - alpha;
		fd->b1 =  -2*cs;
		fd->b2 =   1 + alpha;
		fd->a0 =   1 + alpha;
		fd->a1 =  -2*cs;
		fd->a2 =   1 - alpha;
		break;
	case FTPEAKING:
		fd->b0 =   1 + alpha*A;
		fd->b1 =  -2*cs;
		fd->b2 =   1 - alpha*A;
		fd->a0 =   1 + alpha/A;
		fd->a1 =  -2*cs;
		fd->a2 =   1 - alpha/A;
		break;
	case FTLSHELF:
		fd->b0 =    A*( (A+1) - (A-1)*cs + 2*sqrt(A)*alpha );
		fd->b1 =  2*A*( (A-1) - (A+1)*cs                   );
		fd->b2 =    A*( (A+1) - (A-1)*cs - 2*sqrt(A)*alpha );
		fd->a0 =        (A+1) + (A-1)*cs + 2 *sqrt(A)*alpha  ;
		fd->a1 =   -2*( (A-1) + (A+1)*cs                   );
		fd->a2 =        (A+1) + (A-1)*cs - 2 *sqrt(A)*alpha;
		break;
	case FTHSHELF:
		fd->b0 =    A*( (A+1) + (A-1)*cs + 2*sqrt(A)*alpha );
		fd->b1 = -2*A*( (A-1) + (A+1)*cs                   );
		fd->b2 =    A*( (A+1) + (A-1)*cs - 2*sqrt(A)*alpha );
		fd->a0 =        (A+1) - (A-1)*cs + 2 *sqrt(A)*alpha;
		fd->a1 =    2*( (A-1) - (A+1)*cs                   );
		fd->a2 =        (A+1) - (A-1)*cs - 2 *sqrt(A)*alpha;
		break;
	}
	//normalizace
	fd->b0=fd->b0/fd->a0;
	fd->b1=fd->b1/fd->a0;
	fd->b2=fd->b2/fd->a0;
	fd->a1=fd->a1/fd->a0;
	fd->a2=fd->a2/fd->a0;
	fd->a0=1;
	return FL_OK;
}

//vypocet modulu odezvy filtru
float FlAmp(filter f, unsigned int frequency) {
	//omega
	float o=(float)frequency/(float)f.f0;
	//        float A=dBToA(f.dbGain);

	switch (f.ftype) {
	case FTGAIN:
		return dBToK(f.dbGain);
	case FTLPF:
		//H(s) = 1 / (s^2 + s/Q + 1)
		return fabs(f.Q) *sqrt(1/
		                       (
		                               pow (o,4) * f.Q * f.Q-o * o * (2 * f.Q * f.Q-1)+f.Q * f.Q
		                       )
		);
	case FTHPF:
		return fabs(f.Q) * o * o *sqrt(1/
		                               (
		                                       pow (o,4) * f.Q * f.Q-o * o * (2 * f.Q * f.Q-1)+f.Q * f.Q
		                               )
		);

	case FTBPF:
		return fabs(f.Q) * o *sqrt(1/
		                           (
		                                   pow (o,4) * f.Q * f.Q-o * o * (2 * f.Q * f.Q-1)+f.Q * f.Q
		                           )
		);
	case FTNOTCH:
		if (o==1) {o=1.000001;};
		return fabs(f.Q*(o*o-1)) *sqrt(1/
		                               (
		                                       pow (o,4) * f.Q * f.Q-o * o * (2 * f.Q * f.Q-1)+f.Q * f.Q
		                               )
		);
	case FTAPF:
		return 1;
		break;
	case FTPEAKING:
		if (o==1) {o=1.0001;};
		return fabs(dBToA(f.dbGain)) *sqrt(
		               (
		                       dBToA (f.dbGain) * dBToA (f.dbGain) * o * o+
		                       (o * o-1) * (o * o-1) * f.Q * f.Q
		               )
		               /
		               (
		                       dBToA (f.dbGain) * dBToA (f.dbGain) * (o * o-1) * (o * o-1) *
		                       f.Q * f.Q+o * o
		               )
		);

	case FTLSHELF:
		if (o==1) {o=1.0001;};
		return fabs(dBToA(f.dbGain)) *sqrt(
		               (
		                       dBToA (f.dbGain) * dBToA (f.dbGain) * f.Q * f.Q-
		                       dBToA (f.dbGain) * o * o * (2 * f.Q * f.Q-1)+
		                       o * o * o * o * f.Q * f.Q
		               )
		               /
		               (
		                       dBToA (f.dbGain) * dBToA (f.dbGain) * o * o * o * o * f.Q * f.Q-
		                       dBToA (f.dbGain) * o * o * (2 * f.Q * f.Q-1)+f.Q * f.Q
		               )
		);

	case FTHSHELF:
		if (o==1) {o=1.0001;};
		return fabs(dBToA(f.dbGain)) *sqrt(
		               (
		                       dBToA (f.dbGain) * dBToA (f.dbGain) * o * o * o * o * f.Q * f.Q-
		                       dBToA (f.dbGain) * o * o * (2 * f.Q * f.Q-1)+f.Q * f.Q
		               )
		               /
		               (
		                       dBToA (f.dbGain) * dBToA (f.dbGain) * f.Q * f.Q-
		                       dBToA (f.dbGain) * o * o * (2 * f.Q * f.Q-1)+
		                       o * o * o * o * f.Q * f.Q
		               )
		);

	}
	return 0;
}

//vypocet faze odezvy
float FlPha(filter f, unsigned int frequency) {
	//omega
	float o=(float)frequency/(float)f.f0;
	float A=dBToA(f.dbGain);
	if (A==1) {A=1.0001;};


	switch (f.ftype) {
	case FTGAIN:
		return 0;
		break;
	case FTLPF:
		return
		       -Sign(o*(o*o*o*o*f.Q*f.Q-o*o*(2*f.Q*f.Q-1)+f.Q*f.Q)*f.Q)*M_PI/2
		       -atan((o*o-1)*f.Q/o)
		;
		break;
	case FTHPF:
		return
		       +Sign(o*(o*o*o*o*f.Q*f.Q-o*o*(2*f.Q*f.Q-1)+f.Q*f.Q)*f.Q)*M_PI/2
		       -atan((o*o-1)*f.Q/o)
		;                        break;
	case FTBPF:
		if (o==1) {o=1.0001;};
		return
		       -Sign(o*(o*o-1)*(o*o*o*o*f.Q*f.Q-o*o*(2*f.Q*f.Q-1)+f.Q*f.Q))*M_PI/2
		       +atan(o/((o*o-1)*f.Q))
		;
		break;
	case FTNOTCH:
		if (o==1) {o=1.000001;};
		return
		       +Sign(o*(o*o-1)*(o*o*o*o*f.Q*f.Q-o*o*(2*f.Q*f.Q-1)+f.Q*f.Q))*M_PI/2
		       -atan(((o*o-1)*f.Q)/o)
		;
		break;
	case FTAPF:
		if (o==1) {o=1.0001;};
		return
		       +Sign(o*(o*o-1)*(o*o*o*o*f.Q*f.Q-o*o*(2*f.Q*f.Q-1)+f.Q*f.Q))*M_PI/2
		       -atan((o*o*o*o*f.Q*f.Q-o*o*(2*f.Q*f.Q+1)+f.Q*f.Q)/
		             (2*o*(o*o-1)*f.Q))
		;
		break;
	case FTPEAKING:
		if (o==1) {o=1.0001;};
		return
		       -Sign(
		               A*(A*A-1)*o*(o*o-1)
		       )*M_PI/2
		       +atan(
		               (A*(o*o*o*o*f.Q*f.Q-o*o*(2*f.Q*f.Q-1)+f.Q*f.Q))
		               /
		               ((A*A-1)*o*(o*o-1)*f.Q)
		       )
		;
		break;
	case FTLSHELF:
		if (o==1) {o=1.0001;};
		return
		       -Sign(
		               sqrt(A)*(A-1)*(A*A*o*o*o*o*f.Q*f.Q-A*o*o*(2*f.Q-1)+f.Q*f.Q)*o*f.Q
		       )*M_PI/2
		       -atan(
		               (A*A*o*o*f.Q*f.Q-A*(o*o*o*o*f.Q*f.Q+o*o+f.Q*f.Q)+o*o*f.Q*f.Q)
		               /
		               (sqrt(A)*(A-1)*o*(o*o+1)*f.Q)
		       )
		;
		break;
	case FTHSHELF:
		if (o==1) {o=1.0001;};
		return
		       +Sign(
		               sqrt(A)*(A-1)*(A*A*f.Q*f.Q-A*o*o*(2*f.Q*f.Q-1)+o*o*o*o*f.Q*f.Q)*o*f.Q
		       )*M_PI/2
		       +atan(
		               (A*A*o*o*f.Q*f.Q-A*(o*o*o*o*f.Q*f.Q+o*o+f.Q*f.Q)+o*o*f.Q*f.Q)
		               /
		               (sqrt(A)*(A-1)*o*(o*o+1)*f.Q)
		       )
		;
		break;
	}
	return 0;
}



//---------------------------------------------------------------------------


