/*! \file glcd.c \brief Graphic LCD API functions. */
//*****************************************************************************
//
// File Name	: 'glcd.c'
// Title		: Graphic LCD API functions
// Author		: Pascal Stang - Copyright (C) 2002
// Date			: 5/30/2002
// Revised		: 5/30/2002
// Version		: 0.5
// Target MCU	: Atmel AVR
// Editor Tabs	: 4
//
// NOTE: This code is currently below version 1.0, and therefore is considered
// to be lacking in some functionality or documentation, or may not be fully
// tested.  Nonetheless, you can expect most functions to work.
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************

#ifndef WIN32
// AVR specific includes
        #include <avr/io.h>
        #include <avr/pgmspace.h>
#endif

#include "glcd.h"

// include hardware support
#include "ks0108.h"
// include fonts
#include "font5x7.h"
#include "fontprop.h"
#include "fontgr.h"

// graphic routines

unsigned char FontP=0;

void glcdFont(u08 x)
{
	FontP=x;
}

// set dot
void glcdSetDot(u08 x, u08 y)
{
	unsigned char temp;

	glcdSetAddress(x, y/8);
	temp = glcdDataRead();  // dummy read
	temp = glcdDataRead();  // read back current value
	glcdSetAddress(x, y/8);
	glcdDataWrite(temp | (1 << (y % 8)));

	glcdStartLine(0);
}

// clear dot
void glcdClearDot(u08 x, u08 y)
{
	unsigned char temp;

	glcdSetAddress(x, y/8);
	temp = glcdDataRead();  // dummy read
	temp = glcdDataRead();  // read back current value
	glcdSetAddress(x, y/8);
	glcdDataWrite(temp & ~(1 << (y % 8)));

	glcdStartLine(0);
}

// kravina, ale pekna
void glcdTurnBlack(unsigned int cycles)
{
	glcdClearDisp();
	for (; cycles>0; cycles--) {
		glcdSetDot(random()%(GLCD_XPIXELS-1),random()%(GLCD_YPIXELS-1));
	}
}

void glcdTurnBlack2(unsigned int cycles)
{
	glcdClearDisp();
	for (; cycles>0; cycles--) {
		glcdSetDot(random()%(GLCD_XPIXELS-1),random()%(GLCD_YPIXELS-1));
		glcdSetDot(random()%(GLCD_XPIXELS-1),random()%(GLCD_YPIXELS-1));
		glcdClearDot(random()%(GLCD_XPIXELS-1),random()%(GLCD_YPIXELS-1));
	}
}



// vykresli progressbar
void glcdProgress(unsigned char x,unsigned char y,unsigned char length,unsigned char proc)
{
	unsigned char j,k;
	glcdRectangle(x,y,4,length+1);
	for (j=x+1; j<x+length; j++) {
		if (proc>(j-x)) {
			for (k=1; k<3; k++) {
				glcdSetDot(j,y+k);
			}
		} else {
			for (k=1; k<3; k++) {
				glcdClearDot(j,y+k);
			}
		}
	}
}

// draw line
void glcdLine(u08 x1, u08 y1, u08 x2, u08 y2)
{
	unsigned char j,l;
	unsigned char k=y2-y1;
	unsigned long p,p2;
	if (x1!=x2) {
		for (j = x1; j<=x2; j++) {
			p=((unsigned long)(j-x1)*(unsigned long)k)/(unsigned long)(x2-x1);
			p2=((unsigned long)(j+1-x1)*(unsigned long)k)/(unsigned long)(x2-x1);
			for (l=y1+p; l<=y1+p2; l++) {
				glcdSetDot(j,l);
			}
		}

	} else {
		for (unsigned char y=y1; y<=y2; y++) {
			glcdSetDot(x1,y);
		}
	}
};

// draw line
void glcdDottedLine(u08 x1, u08 y1, u08 x2, u08 y2)
{
	unsigned char j,l;
	unsigned char k=y2-y1;
	unsigned long p,p2;
	if (x1!=x2) {
		for (j = x1; j<=x2; j++) {
			p=((unsigned long)(j-x1)*(unsigned long)k)/(unsigned long)(x2-x1);
			p2=((unsigned long)(j+1-x1)*(unsigned long)k)/(unsigned long)(x2-x1);
			for (l=y1+p; l<=y1+p2; l++) {
				if ((l+j)%2) {glcdSetDot(j,l);}
			}
		}
	} else {
		for (unsigned char y=y1; y<=y2; y++) {
			if ((y)%2) {glcdSetDot(x1,y);}
		}
	}
};


void glcdClrLine(u08 x1, u08 y1, u08 x2, u08 y2)
{
	unsigned char j,l;
	unsigned char k=y2-y1;
	unsigned long p,p2;
	if (x1!=x2) {
		for (j = x1; j<=x2; j++) {
			p=((unsigned long)(j-x1)*(unsigned long)k)/(unsigned long)(x2-x1);
			p2=((unsigned long)(j+1-x1)*(unsigned long)k)/(unsigned long)(x2-x1);
			for (l=y1+p; l<=y1+p2; l++) {
				glcdClearDot(j,l);
			}
		}

	} else {
		for (unsigned char y=y1; y<=y2; y++) {
			glcdClearDot(x1,y);
		}
	}
};

// draw rectangle
void glcdRectangle(u08 x, u08 y, u08 a, u08 b)
{
	unsigned char j;

	for (j = 0; j < a; j++) {
		glcdSetDot(x, y + j);
		glcdSetDot(x + b - 1, y + j);
	}
	for (j = 0; j < b; j++)       {
		glcdSetDot(x + j, y);
		glcdSetDot(x + j, y + a - 1);
	}
}

void glcdClrRectangle(u08 x, u08 y, u08 a, u08 b)
{
	unsigned char j;

	for (j = 0; j < a; j++) {
		glcdClearDot(x, y + j);
		glcdClearDot(x + b - 1, y + j);
	}
	for (j = 0; j < b; j++)       {
		glcdClearDot(x + j, y);
		glcdClearDot(x + j, y + a - 1);
	}
}


void glcdClearDisp(void)
{
	unsigned int xpoc,line;
	for (line=0; line<GLCD_TEXT_LINES; line++) {
		glcdSetAddress(0,line);
		for (xpoc=0; xpoc<GLCD_XPIXELS; xpoc++) {
			glcdDataWrite(0);
		};
	};
};

void glcdmot(u08 xcenter,u08 ycenter,u08 faze)
{
	glcdCircle(xcenter,ycenter,8);
	signed char j;
	for (j=-5; j<=5; j++) {
		glcdClearDot(xcenter+j,ycenter+j);
		glcdClearDot(xcenter+j,ycenter);
		glcdClearDot(xcenter,ycenter+j);
		glcdClearDot(xcenter-j,ycenter+j);
		if (faze==2) {
			glcdSetDot(xcenter,ycenter+j);
		}
		if (faze==4) {
			glcdSetDot(xcenter+j,ycenter);
		}
	}
	for (j=-4; j<=4; j++) {
		if (faze==1) {
			glcdSetDot(xcenter+j,ycenter+j);
		}
		if (faze==3) {
			glcdSetDot(xcenter-j,ycenter+j);
		}
	}

}

// draw circle
void glcdCircle(u08 xcenter, u08 ycenter, u08 radius)
{
	int tswitch, y, x = 0;
	unsigned char d;

	d = ycenter - xcenter;
	y = radius;
	tswitch = 3 - 2 * radius;
	while (x <= y) {
		glcdSetDot(xcenter + x, ycenter + y);     glcdSetDot(xcenter + x, ycenter - y);
		glcdSetDot(xcenter - x, ycenter + y);     glcdSetDot(xcenter - x, ycenter - y);
		glcdSetDot(ycenter + y - d, ycenter + x); glcdSetDot(ycenter + y - d, ycenter - x);
		glcdSetDot(ycenter - y - d, ycenter + x); glcdSetDot(ycenter - y - d, ycenter - x);

		if (tswitch < 0) tswitch += (4 * x + 6);
		else {
			tswitch += (4 * (x - y) + 10);
			y--;
		}
		x++;
	}
}

void glcdClrCircle(u08 xcenter, u08 ycenter, u08 radius)
{
	int tswitch, y, x = 0;
	unsigned char d;

	d = ycenter - xcenter;
	y = radius;
	tswitch = 3 - 2 * radius;
	while (x <= y) {
		glcdClearDot(xcenter + x, ycenter + y);     glcdClearDot(xcenter + x, ycenter - y);
		glcdClearDot(xcenter - x, ycenter + y);     glcdClearDot(xcenter - x, ycenter - y);
		glcdClearDot(ycenter + y - d, ycenter + x); glcdClearDot(ycenter + y - d, ycenter - x);
		glcdClearDot(ycenter - y - d, ycenter + x); glcdClearDot(ycenter - y - d, ycenter - x);

		if (tswitch < 0) tswitch += (4 * x + 6);
		else {
			tswitch += (4 * (x - y) + 10);
			y--;
		}
		x++;
	}
}

// text routines

// write a character at the current position
void glcdWriteChar(unsigned char c)
{
	u08 i = 0;

	unsigned char ddata,ldata;
	ldata=0;
	if ( /*glcd_font*/ 0&FONT_BOLD) {
		for(i=0; i<10; i++)
		{
			glcdDataWrite(PRG_RDB(&Font5x7[((c - 0x20) * 5) + i/2]));
		}
	} else {

		for(i=0; i<5; i++)
		{
			ddata=PRG_RDB(&Font5x7[((c - 0x20) * 5) + i]);
			if (0 /*glcd_font*/&FONT_UNDERLINE) {ddata|=128;}
			glcdDataWrite(ddata|ldata);
			if (0 /*glcd_font*/&FONT_BOLD2) {
				ldata=ddata;
			}
		}
	}

	// write a spacer line
	if (0 /*glcd_font*/&FONT_CURSOR) {
		glcdDataWrite(0xFF);
	} else {
		glcdDataWrite((0 /*glcd_font*/&FONT_UNDERLINE)|ldata);
	}
	// unless we're at the end of the display
	//if(xx == 128)
	//	xx = 0;
	//else
	//	glcdWriteData(0x00);

	//cbi(GLCD_Control, GLCD_CS1);
	//cbi(GLCD_Control, GLCD_CS2);
	glcdStartLine(0);
}

void glcdWriteBoldChar(unsigned char c)
{
	u08 i = 0;

	for(i=0; i<10; i++)
	{
		glcdDataWrite(PRG_RDB(&Font5x7[((c - 0x20) * 5) + i/2]));
	}

	// write a spacer line
	glcdDataWrite(0x00);
	glcdDataWrite(0x00);
	// unless we're at the end of the display
	//if(xx == 128)
	//	xx = 0;
	//else
	//	glcdWriteData(0x00);

	//cbi(GLCD_Control, GLCD_CS1);
	//cbi(GLCD_Control, GLCD_CS2);
	glcdStartLine(0);
}

void glcdWriteBigChar(unsigned char c)
{
	unsigned char i;

/*	for (i=0;i<16;i++) {
   	  glcdDataWrite(PRG_RDB((&font16x8a+c*16+i])));
   	}  */
}

void glcdWriteBMP(unsigned char x,unsigned char line, uint16_t obr)
{
	unsigned char xpoc,ypoc;
	//nacti rozmery
	unsigned char xroz=PRG_RDB(obr);
	unsigned char yroz=PRG_RDB(obr+1);

	for (ypoc=0; ypoc<(yroz/8); ypoc++) {
		glcdSetAddress(x,line+ypoc);
		for (xpoc=0; xpoc<xroz; xpoc++) {
			glcdDataWrite(PRG_RDB(obr+2+xpoc+ypoc*xroz));
		}
	}
}

void glcdWriteCharGr(u08 grCharIdx)
{
	u08 idx;
	u08 grLength;
	u08 chard;
	u16 grStartIdx = 0;

	grCharIdx-=0x20;

	// get starting index of graphic bitmap
	for(idx=0; idx<grCharIdx; idx++)
	{
		// add this graphic's length to the startIdx
		// to get the startIdx of the next one
		grStartIdx += PRG_RDB(FontPropLen+idx);
	}

	grLength = PRG_RDB(FontPropLen+grCharIdx);

	unsigned char boldd=0;
	// write the lines of the desired graphic to the display
	for(idx=0; idx<grLength; idx++)
	{
		// write the line
		chard=PRG_RDB(FontProp+(grStartIdx+1)+idx-1);
		if (FontP&FP_BOLD) {
			if (boldd) {chard|=boldd;};
			boldd=PRG_RDB(FontProp+(grStartIdx+1)+idx-1);
		}
		if (FontP&FP_ITALIC) {
			unsigned char pom=boldd;
			boldd=chard&0x1F;
			chard&=0xE0;
			chard|=pom;
		}
		if (FontP&FP_UNDERLINE) {chard|=0x80;};
		if (FontP&FP_NEGATIVE) {
			glcdDataWrite(~chard);
		} else {
			glcdDataWrite(chard);
		}
	}
	if (boldd) {
		if (FontP&FP_NEGATIVE) {
			glcdDataWrite(~boldd);
		} else {
			glcdDataWrite(boldd);
		}
	}
	if (FontP&FP_NEGATIVE) {
		glcdDataWrite(0xFF);
	} else {
		glcdDataWrite(0x00);
	}
}




void glcdPutStr(unsigned char * data)
{
	while (*data) {
		glcdWriteChar(*data);
		data++;
	}
}
