#include <avr/io.h>
#include <avr/interrupt.h>

#include <string.h>
#include <stdio.h>
#include <avr/pgmspace.h>

#include "hwdef.h"
#include "intf.h"
#include "a2d.h"
#include "i2c.h"

void DelayUs (unsigned char c)
{
	int i;
	i=16*c;
	i=(i >> 2);
	while (i) {
		i--; asm ("nop");
	}
}

void DelayMs (unsigned int c)
{
	while (c) {
		c--;
		DelayUs(250);
		DelayUs(250);
		DelayUs(250);
		DelayUs(250);
	}
}




//funkce zasila data multimedialni klavesnice do obvodu PCM

//PORTD
//2 .. CLK
//3 .. DATA
//4 .. LATCH (MS)
//5 .. LED
//0 .. /POSLATDESKRIPTOR

unsigned char PCMlatchreg=0;

void PCMSetPins(unsigned char db) {
	PCMlatchreg=db;
	//odeslat na TAS SLAVE	
	unsigned char data[5]={0xEE,0,0,0,db};
	i2cMasterSend(TASS, 5, data);		
}

void PCMInit(void) {	
	//odeslat na TAS SLAVE	
	unsigned char data[5]={0xEF,0x00,0x02,0x6E,0x6D};
	i2cMasterSend(TASS, 5, data);		
}


void PCMSendByte(unsigned char data) {
 unsigned char i=8;
 while (i) {
  if (data&128) {
   PCMlatchreg|=1;
  } else {
   PCMlatchreg&=~1;
  }
  
  PCMSetPins(PCMlatchreg);
  DelayUs(20);
  
  PCMlatchreg|=8;
  
  PCMSetPins(PCMlatchreg);
  DelayUs(200);
  
  PCMlatchreg&=~8;
  
  PCMSetPins(PCMlatchreg);
  DelayUs(200);
  
  data=data<<1;
  i--;
 }
}

void PCMSendHID(unsigned char delta) {
 PCMlatchreg&=~8;
 PCMSetPins(PCMlatchreg);
 
 PCMlatchreg&=~4;
 PCMSetPins(PCMlatchreg);
 
 PCMSendByte(0);
 PCMSendByte(delta);
 
 PCMlatchreg|=4;
 PCMSetPins(PCMlatchreg);
 
 DelayMs(10);
 
 PCMlatchreg&=~8;
 PCMSetPins(PCMlatchreg);
 
 PCMlatchreg&=~4;
 PCMSetPins(PCMlatchreg);
 
 PCMSendByte(0);
 PCMSendByte(0);
 
 PCMlatchreg|=4;
 PCMSetPins(PCMlatchreg);
}








//funkce inicializuje DRC subsystem v obvodu TAS
void InitDRC(unsigned char tasid) {
	TASSenddB(tasid,0x77,g0dB,0); // mix v to k
	TASSenddB(tasid,0x76,g0dB,0); // mix u to i
	TASSenddB(tasid,0x78,g0dB,0); // mix w to m

	//casove konstanty

	{
		unsigned char lddata[9]={0xB1,0x00,0x00,0x03,0x46,0x00,0x7F,0xFC,0xB9};

		//ae pro DRC CH1/2
		i2cMasterSendNI(tasid, 9, lddata);
		//ae pro DRC CH3
		lddata[0]=0xB6;
		i2cMasterSendNI(tasid, 9, lddata);
	}

	{
		unsigned char lddata[9+8]={0xB5,0x00,0x19,0x99,0x99,0x00,0x66,0x66,0x66,0x00,0x00,0x03,0x46,0x00,0x7F,0xFC,0xB9};

		//aa, ad pro DRC CH1/2
		i2cMasterSendNI(tasid, 9+8, lddata);
		//aa, ad pro DRC CH3
		lddata[0]=0xBA;
		i2cMasterSendNI(tasid, 9+8, lddata);
	}

	{
		unsigned char lddata[9]={0x7C,0x00,0x80,0x00,0x00,0x00,0x00,0x00,0x00};

		//zapnuti DRC
		i2cMasterSendNI(tasid, 9, lddata);
		lddata[0]=0x7D;
		i2cMasterSendNI(tasid, 9, lddata);
		lddata[0]=0x7E;
		i2cMasterSendNI(tasid, 9, lddata);

	}

	//naplneni DRC daty
	//vypocet DRC dat

	//unsigned short body



}


void RefreshVolume(void) {
	unsigned char outvol = 255-GetVolume()/4;
	for (unsigned char i=0; i<8; i++) {
		unsigned char outvolt=outvol;
		switch (i) {
			case 0:/* out D */ if (!(volset&0x20)) {outvolt=0;} break;
			case 1:/* out D */ if (!(volset&0x20)) {outvolt=0;} break;
			case 2:/* out C */ if (!(volset&0x08)) {outvolt=0;} break;
			case 3:/* out C */ if (!(volset&0x10)) {outvolt=0;} break;
			case 4:/* out B */ if (!(volset&0x04)) {outvolt=0;} break;
			case 5:/* out B */ if (!(volset&0x04)) {outvolt=0;} break;
			case 6:/* out A */ if (!(volset&0x01)) {outvolt=0;} break;
			case 7:/* out A */ if (!(volset&0x02)) {outvolt=0;} break;
		}
		CSSetVolume(i,outvolt);
	}
}

unsigned char flodata[8];

#include <math.h>

void FloatTo523(float f) {
	unsigned char inv = 0;
	if (f<0.0) {inv=1; f=-f;}


	f *= (float)((long)1<<23);

	unsigned long part = f;
	part=part&0x07FFFFFF;

	flodata[0]=part>>24;
	flodata[1]=(part>>16)&0x000000FF;
	flodata[2]=(part>>8)&0x000000FF;
	flodata[3]=(part)&0x000000FF;

	if (inv) {
		flodata[0]=~flodata[0];
		flodata[1]=~flodata[1];
		flodata[2]=~flodata[2];
		flodata[3]=~flodata[3];
	}


}

void FloatTo2523(float f) {
	unsigned char inv = 0;
	if (f<0.0) {inv=1; f=-f;}


	f *= (float)((long)1<<23);

	unsigned long long part = f;
	part=part&0x07FFFFFFFFFF;

	flodata[0]=0;
	flodata[1]=0;

	flodata[2]=(part>>40)&0x000000FF;
	flodata[3]=(part>>32)&0x000000FF;
	flodata[4]=(part>>24)&0x000000FF;
	flodata[5]=(part>>16)&0x000000FF;
	flodata[6]=(part>>8)&0x000000FF;
	flodata[7]=(part)&0x000000FF;

	if (inv) {
		for (unsigned char i=2; i<8; i++)
			flodata[i]=~flodata[i];
	}


}


void LoadDRC(unsigned char id) {
	//id odpovida pismenu


	unsigned char tasid=TASM;
	if (id<2) {tasid=TASM;} else {tasid=TASS;}

	//vypocet koeficientu
	//tresholdy
	unsigned char trdata[1+2*8];
	trdata[0]=(id%2) ? 0xB2 : 0xB7;

	float ft;
	ft = (80.0+48.0-(float)drcset[id].treshold[0])/6.0206;
	//v ft jsou data pro treshold 0
	FloatTo2523(ft);
	for (unsigned char i=0; i<8; i++) {
		trdata[1+0+i]=flodata[0+i];
	}

	ft = (80.0+48.0-(float)drcset[id].treshold[1])/6.0206;
	//v ft jsou data pro treshold 1
	FloatTo2523(ft);
	for (unsigned char i=0; i<8; i++) {
		trdata[1+8+i]=flodata[0+i];
	}


	i2cMasterSendNI(tasid, 1+2*8, trdata);

	//offsety

	unsigned char ofdata[1+2*8];
	ofdata[0]=(id%2) ? 0xB4 : 0xB9;

	ft = ((50.0-(float)drcset[id].offset[0]+24.0824)/6.0206);
	//v ft jsou data pro offset 0
	FloatTo2523(ft);
	for (unsigned char i=0; i<8; i++) {
		ofdata[1+i]=flodata[0+i];
	}

	ft = ((50.0-(float)drcset[id].offset[1]+24.0824)/6.0206);
	//v ft jsou data pro offset 1
	FloatTo2523(ft);
	for (unsigned char i=0; i<8; i++) {
		ofdata[1+8+i]=flodata[0+i];
	}


	i2cMasterSendNI(tasid, 1+2*8, ofdata);



	unsigned char slopedata[1+3*4];
	slopedata[0]=(id%2) ? 0xB3 : 0xB8;


	//slope
	ft = (float)drcset[id].slopeL/10.0  - 1.0;
	//slope L
	FloatTo523(ft);
	for (unsigned char i=0; i<4; i++) {
		slopedata[1+i]=flodata[0+i];
	}

	ft = (((float)drcset[id].treshold[1]+(float)drcset[id].offset[1]-50.0-((float)drcset[id].treshold[0]+(float)drcset[id].offset[0]-50.0)))/((float)drcset[id].treshold[1]-(float)drcset[id].treshold[0]);
	ft = ft  - 1.0;

	FloatTo523(ft);
	for (unsigned char i=0; i<4; i++) {
		slopedata[5+i]=flodata[0+i];
	}

	ft = (float)drcset[id].slopeH/10.0  - 1.0;

	//slope mezi
	FloatTo523(ft);
	for (unsigned char i=0; i<4; i++) {
		slopedata[9+i]=flodata[0+i];
	}


	//slope H


	i2cMasterSendNI(tasid, 1+3*4, slopedata);
}


//init portu + cteni hlasitosti
void InitPorts(void) {
	DDRA=0x8C;
	PORTA=0x73;

	DDRB=0xC0;
	PORTB=0x3F;

	DDRE=0x04;
	PORTE=0x04;

	DDRF=0x00;
	PORTF=0x0F;

	a2dInit();
	a2dSetChannel(4);
	a2dStartConvert();
}


//ovladani predniho panelu





unsigned char GetButtons(void) {
	unsigned char data[1];
	i2cMasterReceive(PPEXP, 1, data);
	data[0]=(~data[0])&0xF6;
	if (!(PINF&1)) {data[0]|=0x01;};
	return data[0];
}

unsigned char LedStat=0;


void SetLed(unsigned char id) {
	unsigned char data[1];
	LedStat|=id;
	data[0]=(~LedStat)|4|2;
	i2cMasterSendNI(PPEXP, 1, data);
}

void ResetLed(unsigned char id) {
	unsigned char data[1];
	LedStat&=~id;
	data[0]=(~LedStat)|4|2;
	i2cMasterSendNI(PPEXP, 1, data);
}

volatile unsigned char rotval=0;
volatile unsigned char rotmax=100;
volatile unsigned char rotsa=0;

void SetRotary(unsigned char max, unsigned char value) {
	rotval=value;
	rotmax=max;
}

unsigned char GetRotary(void) {
	return rotval;
}

void ProcessRotary(void) {
	unsigned char rotstat=rotsa;
	rotsa=0;
	if (PINF&2) {rotsa|=1;};
	if (PINF&4) {rotsa|=2;};
	//kvadraturni enkoder

	if (((rotstat&1)!=(rotsa&1))!=((rotstat&2)!=(rotsa&2))) {
		if ((((rotsa&1)!=(rotstat&1))&&((rotsa&2)!=(rotsa&1)) )||(
		            ((rotsa&2)!=(rotstat&2))&&((rotsa&2)==(rotsa&1)))
		) {
			if (rotval<rotmax) {rotval++;};
		} else {
			if (rotval) {rotval--;};
		}
	}

}

void TASMute(unsigned char tasid) {
	unsigned char data[5]={0xF0,0,0,0,0x07};
	i2cMasterSendNI(tasid, 5, data);
	DelayMs(200);
}

void TASUnMute(unsigned char tasid) {
	unsigned char data[5]={0xF0,0,0,0,0};
	i2cMasterSendNI(tasid, 5, data);
	DelayMs(200);
}

#define PRG_RDB(addr) (pgm_read_byte(addr))

void TASSendCoef(unsigned char tasid, unsigned char parameter, fdata * fd) {
	unsigned char data[1+5*4]={0x00,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

	data[0]=parameter;
	float coef=0.0;

	for (unsigned char cid=0; cid<5; cid++) {
		if (cid==0) {coef=-fd->a1;};
		if (cid==1) {coef=-fd->a2;};
		if (cid==2) {coef=-fd->b0;};
		if (cid==3) {coef=-fd->b1;};
		if (cid==4) {coef=-fd->b2;};

		if (coef<0.0) {
			coef=-coef;
			data[1+4*cid]=0x08;
		}

		float vaha=16.0;
		unsigned char bit=27;

		do {
			vaha=vaha/2.0;
			bit--;
			if (coef>vaha) {
				if (!(data[1+4*cid]&0x08)) {
					data[4-(bit/8)+4*cid]|=(1<<(bit%8));
				}
				coef-=vaha;
			} else {
				if ((data[1+4*cid]&0x08)) {
					data[4-(bit/8)+4*cid]|=(1<<(bit%8));
				}
			}
		} while (bit);
	}

	i2cMasterSendNI(tasid, 1+5*4, data);
	DelayUs(200);
}


void TASSenddB(unsigned char tasid, unsigned char parameter,unsigned char dBVal,unsigned char invert) {
	unsigned char data[5]={0xF0,0,0,0,0};

	data[0]=parameter;
	data[1]=PRG_RDB(VolTab+dBVal*4);
	data[2]=PRG_RDB(VolTab+dBVal*4+1);
	data[3]=PRG_RDB(VolTab+dBVal*4+2);
	data[4]=PRG_RDB(VolTab+dBVal*4+3);

	if (invert) {
		data[4]=~data[4];
		data[3]=~data[3];
		data[2]=~data[2];
		data[1]=~data[1];
	}

	i2cMasterSendNI(tasid, 5, data);
	DelayUs(200);
}


void TASSetupMainMix(unsigned char id) {
	unsigned char tasid;
	unsigned char voll;
	unsigned char volr;
	unsigned char volrt;
	unsigned char vollt;

	if (id) {tasid=TASS;} else {tasid=TASM;};
	//nastaveni TASu

	TASSenddB(tasid,0xCA,g0dB,0);
	TASSenddB(tasid,0xCD,g0dB,0);
	TASSenddB(tasid,0x01,MUTEdB,0);
	TASSenddB(tasid,0x08,MUTEdB,0);


	//nastaveni mixu

	//INP1

	for (unsigned char i=0; i<4; i++) {
		vollt=volrt=MUTEdB;
		unsigned char voicecan = 0;
		//hlasitost
		voll=volr=mainmix[id].inpmix[i].volume;
		//balance
		if (mainmix[id].inpmix[i].balance>128) {
			voll=voll>(mainmix[id].inpmix[i].balance-128) ? voll-(mainmix[id].inpmix[i].balance-128) : 0;
		} else {
			volr=volr>(128-mainmix[id].inpmix[i].balance) ? volr-(128-mainmix[id].inpmix[i].balance) : 0;
		}
		//mono, ch. swap
		if (mainmix[id].inpmix[i].flags&REZ_MONO) {
			if (voll>6) {voll-=6;}
			vollt=voll;
			if (volr>6) {volr-=6;}
			volrt=volr;
			if (mainmix[id].inpmix[i].flags&CHANNEL_SWAP) {
				voicecan=1;
			}
		} else
		if (mainmix[id].inpmix[i].flags&CHANNEL_SWAP) {
			vollt=voll;
			volrt=volr;
			voll=volr=0;
		}

		//phase inv

		//TBD


		if (i==0) {
			TASSenddB(tasid,0x04,voll,mainmix[id].inpmix[i].flags&PH_INV_L);
			TASSenddB(tasid,0x0B,volr,mainmix[id].inpmix[i].flags&PH_INV_R || voicecan);
			TASSenddB(tasid,0x05,vollt,mainmix[id].inpmix[i].flags&PH_INV_R);
			TASSenddB(tasid,0x0A,volrt,mainmix[id].inpmix[i].flags&PH_INV_L || voicecan);
		}
		if (i==1) {
			TASSenddB(tasid,0x10,voll,mainmix[id].inpmix[i].flags&PH_INV_L);
			TASSenddB(tasid,0x17,volr,mainmix[id].inpmix[i].flags&PH_INV_R || voicecan);
			TASSenddB(tasid,0x11,vollt,mainmix[id].inpmix[i].flags&PH_INV_R);
			TASSenddB(tasid,0x16,volrt,mainmix[id].inpmix[i].flags&PH_INV_L || voicecan);
		}
		if (i==2) {
			TASSenddB(tasid,0x19,voll,mainmix[id].inpmix[i].flags&PH_INV_L);
			TASSenddB(tasid,0x20,volr,mainmix[id].inpmix[i].flags&PH_INV_R || voicecan);
			TASSenddB(tasid,0x1A,vollt,mainmix[id].inpmix[i].flags&PH_INV_R);
			TASSenddB(tasid,0x1F,volrt,mainmix[id].inpmix[i].flags&PH_INV_L || voicecan);
		}
		if (i==3) {
			TASSenddB(tasid,0x2D,voll,mainmix[id].inpmix[i].flags&PH_INV_L);
			TASSenddB(tasid,0x30,volr,mainmix[id].inpmix[i].flags&PH_INV_R || voicecan);
			TASSenddB(tasid,0x2E,vollt,mainmix[id].inpmix[i].flags&PH_INV_R);
			TASSenddB(tasid,0x2F,volrt,mainmix[id].inpmix[i].flags&PH_INV_L || voicecan);
		}

	}
}





void TASInit(void) {
	APResetOff();
	DelayMs(100);
	TASMute(TASM);
	{
		unsigned char data[5]={0xF9,1,128+64+1,4+8*4,3+16*3};
		i2cMasterSendNI(TASM, 5, data);
		DelayMs(80);
	}
	TASUnMute(TASM);
	TASMute(TASS);
	{
		unsigned char data[5]={0xF9,1,128+1,4+8*4,3+16*3};
		i2cMasterSendNI(TASS, 5, data);
		DelayMs(80);
	}
	TASUnMute(TASS);
	CodecResetOff();
	DelayMs(50);

	//nast. kodeku
	//vzorkovani
	{
		unsigned char data[2]={0x04,73};
		i2cMasterSendNI(CODEC, 2, data);
		DelayMs(10);
	}
	//single ended/dif. nastaveni, transition control
	{
		unsigned char data[3]={0x05,0x0C,64+32};
		i2cMasterSendNI(CODEC, 3, data);
		DelayMs(5);
	}


	//nastav hlasitosti na 0 dB
	{
		unsigned char data[5]={0xF2,0,0x80,0,0};
		i2cMasterSendNI(TASM, 5, data);
		DelayMs(1);
		data[0]=0xF3;
		i2cMasterSendNI(TASM, 5, data);
		DelayMs(1);
		data[0]=0xF4;
		i2cMasterSendNI(TASM, 5, data);
		DelayMs(1);

		//bypass
		data[2]=0x00;
		data[0]=0xCC;
		i2cMasterSendNI(TASM, 5, data);
		DelayMs(1);
		data[0]=0xCF;
		i2cMasterSendNI(TASM, 5, data);
		DelayMs(1);
	}


	//nastav hlasitosti na 0 dB
	{
		unsigned char data[5]={0xF2,0,0x80,0,0};
		i2cMasterSendNI(TASS, 5, data);
		DelayMs(1);
		data[0]=0xF3;
		i2cMasterSendNI(TASS, 5, data);
		DelayMs(1);
		data[0]=0xF4;
		i2cMasterSendNI(TASS, 5, data);
		DelayMs(1);

		//bypass
		data[2]=0x00;
		data[0]=0xCC;
		i2cMasterSendNI(TASS, 5, data);
		DelayMs(1);
		data[0]=0xCF;
		i2cMasterSendNI(TASS, 5, data);
		DelayMs(1);
	}

}

void CSSetVolume(unsigned char out, unsigned char value) {
	unsigned char data[2];
	data[0]=0x08+out;
	data[1]=value;
	i2cMasterSendNI(CODEC, 2, data);
	DelayMs(1);
}

void CSSetIn(unsigned char in) {
	unsigned char data[2]={0x05,0x0C};
	if (in) data[1]=0x0F;
	i2cMasterSendNI(CODEC, 2, data);
	DelayUs(100);
}




void LoadSubMix(unsigned char id) {
	unsigned char tasid;

	if (id) {tasid=TASS;} else {tasid=TASM;};
	//nastaveni TASu



	//nastaveni mixu

	//MONOMIX


	TASSenddB(tasid,0x06,submix[id].gain[0],submix[id].invflags&0x0001);
	TASSenddB(tasid,0x0C,submix[id].gain[1],submix[id].invflags&0x0002);

	TASSenddB(tasid,0x12,submix[id].gain[2],submix[id].invflags&0x0004);
	TASSenddB(tasid,0x18,submix[id].gain[3],submix[id].invflags&0x0008);

	TASSenddB(tasid,0x1E,submix[id].gain[4],submix[id].invflags&0x0010);
	TASSenddB(tasid,0x24,submix[id].gain[5],submix[id].invflags&0x0020);

	TASSenddB(tasid,0xCB,submix[id].gain[6],submix[id].invflags&0x0040);
	TASSenddB(tasid,0xCE,submix[id].gain[7],submix[id].invflags&0x0080);

	TASSenddB(tasid,0xD0,submix[id].gain[8],submix[id].invflags&0x0100);
	TASSenddB(tasid,0xD1,submix[id].gain[9],submix[id].invflags&0x0200);

	TASSenddB(tasid,0x8D,submix[id].gain[10],submix[id].invflags&0x0400);
	TASSenddB(tasid,0x93,submix[id].gain[11],submix[id].invflags&0x0800);
	TASSenddB(tasid,0x99,submix[id].gain[12],0);

	TASSenddB(tasid,0x8C,submix[id].gain[14],submix[id].invflags&(1<<14));
	TASSenddB(tasid,0x92,submix[id].gain[15],submix[id].invflags&(1<<15));
	TASSenddB(tasid,0x98,submix[id].gain[16],0);

	TASSenddB(tasid,0x9E,MUTEdB,0);



}
