#include "intf.h"
#include "ks0108.h"
#include "glcd.h"
#include <stdio.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "hwdef.h"
#include "math.h"

void SaveMainMix(unsigned char id, unsigned char position) {
	if (position--) {
		eeprom_write_block(&mainmix[id],(void *) (position*16),sizeof(t_main_mix_struct));
	}
}

void LoadMainMix(unsigned char id, unsigned char position) {
	if (position--) {
		eeprom_read_block(&mainmix[id],(void *) (position*16),sizeof(t_main_mix_struct));
		TASSetupMainMix(id) ;
		mainmix[1-id].inpmix[inp].flags&=~REZ_EXTCHAN;
		if (mainmix[id].inpmix[inp].flags&REZ_EXTCHAN) {
			mainmix[1-id].inpmix[inp].flags|=REZ_EXTCHAN;
		}

		CSSetIn(mainmix[id].inpmix[2].flags&REZ_EXTCHAN);
	}
}


void LoadDRCFromEE(unsigned char id, unsigned char position) {
	if (position--) {
		eeprom_read_block(&drcset[id],(void *) (500+sizeof(t_drc_struct)*id+4*sizeof(t_drc_struct)*position),sizeof(t_drc_struct));
		LoadDRC(id);
	}
}
void SaveDRCToEE(unsigned char id, unsigned char position) {
	if (position--) {
		eeprom_write_block(&drcset[id],(void *) (500+sizeof(t_drc_struct)*id+4*sizeof(t_drc_struct)*position),sizeof(t_drc_struct));
	}
}

void LoadSubMixFromEE(unsigned char id, unsigned char position) {
	if (position--) {
		eeprom_read_block(&submix[id],(void *) (620+sizeof(t_sub_mix_struct)*id+2*sizeof(t_sub_mix_struct)*position),sizeof(t_sub_mix_struct));
		LoadSubMix(id);
	}
}
void SaveSubMixToEE(unsigned char id, unsigned char position) {
	if (position--) {
		eeprom_write_block(&submix[id],(void *) (620+sizeof(t_sub_mix_struct)*id+2*sizeof(t_sub_mix_struct)*position),sizeof(t_sub_mix_struct));
	}
}



void SaveEq(unsigned char id, unsigned char position) {
	eeprom_write_block((!id) ? &maineq[0] : &maineq[12] /*+12*sizeof(filter)*/,(void *) (128+12*sizeof(filter)*id+sizeof(filter)*24*position),sizeof(filter)*12);
}

void LoadEq(unsigned char id, unsigned char position) {
	eeprom_read_block((!id) ? &maineq[0] : &maineq[12] /*+12*sizeof(filter)*/,(void *) (128+12*sizeof(filter)*id+sizeof(filter)*24*position),sizeof(filter)*12);

	//nacteni mixu
	for (unsigned char i=0; i<12; i++) {
		fdata fd;
		ComputeFilter(&maineq[12*id+i],&fd);

		if (!id) {
			TASSendCoef(TASM, 0x4F+i, &fd);
			DelayUs(200);
			TASSendCoef(TASM, 0x5B+i, &fd);
		} else {

			TASSendCoef(TASS, 0x4F+(i%12), &fd);
			DelayUs(200);
			TASSendCoef(TASS, 0x5B+(i%12), &fd);
		}

	}

}


void SaveSubEq(unsigned char id, unsigned char position) {
	eeprom_write_block((!id) ? &subeq[0] : &subeq[16] /*+12*sizeof(filter)*/,(void *) (735+16*sizeof(filter)*id+sizeof(filter)*32*position),sizeof(filter)*16);
}

void LoadSubEq(unsigned char id, unsigned char position) {
	eeprom_read_block((!id) ? &subeq[0] : &subeq[16] /*+12*sizeof(filter)*/,(void *) (735+16*sizeof(filter)*id+sizeof(filter)*32*position),sizeof(filter)*16);

	//nacteni mixu
	for (unsigned char i=0; i<12; i++) {
		fdata fd;
		ComputeFilter(&subeq[12*id+i],&fd);

		if (!id) {			
			TASSendCoef(TASM, (i<12)?0x67+i:0x48+i-12, &fd);
			DelayUs(200);			
		} else {
			TASSendCoef(TASS,(i<12)?0x67+i:0x48+i-12, &fd);
			DelayUs(200);			
		}

	}

}

unsigned char SelectNumber(unsigned char id, unsigned char max) {
	glcdClearDisp();
	//titulek
	glcdSetAddress(0,LINE1);
	glcdFont(FP_NEGATIVE);
	printf_P(PSTR("    Choose position     "));
	unsigned char volba=id;
	while (!(GetButtons()&(BTNOK|BTNPWR))) {

		while (GetButtons()) {DelayMs(10);};
		glcdSetAddress(15,LINE4);
		for (unsigned char i=1; i<=max; i++) {
			if (volba==i) {
				glcdFont(FP_NEGATIVE);
			} else {
				glcdFont(0);
			}
			printf_P(PSTR(" %d "),i);
		}
		while (!GetButtons()) {RefreshVolume();};
		if (GetButtons()&BTNLEFT&&volba>1) {volba--;};
		if (GetButtons()&BTNRIGHT&&volba<max) {volba++;};
	}
	if (GetButtons()&BTNOK) {
		while (GetButtons()) {DelayMs(10);};
		return volba;
	} else {
		while (GetButtons()) {DelayMs(10);};
		return 0;
	}
}


/*

   //ftype
 #define         FTGAIN          0
 #define         FTLPF           1
 #define         FTHPF           2
 #define         FTBPF           3
 #define         FTNOTCH         4
 #define         FTAPF           5
 #define         FTPEAKING       6
 #define         FTLSHELF        7
 #define         FTHSHELF        8*/

void BiquadEdit(filter * f, unsigned char fi1, unsigned char fi2, unsigned char id) {
	glcdClearDisp();
	//titulek
	glcdSetAddress(0,LINE1);
	glcdFont(FP_NEGATIVE);
	printf_P(PSTR("    Filter editor    "));
	glcdFont(0);

	while ((GetButtons()&(BTNOK|BTNPWR))) {};

	unsigned char rotgain=GetButtons()&BTNROT;


	while (!(GetButtons()&(BTNOK|BTNPWR))) {
		RefreshVolume();

		if (!GetButtons()&BTNROT) {
			if (!rotgain) {
				SetRotary(100,(int)f->dbGain+80);
				rotgain=1;
			} else {
				f->dbGain=GetRotary()-80;
			}
		} else {
			if (rotgain) {
				SetRotary(200,(int)(45.0*log10(f->f0/1.2-15)));
				rotgain=0;
			} else {
				f->f0=(unsigned int)pow(10,GetRotary()/45.0)*1.2+15;
			}
		}

		//volba typu filtru
		glcdSetAddress(15,LINE3);
		switch (f->ftype) {
		case FTGAIN: printf_P(PSTR("Gain")); break;
		case FTLPF: printf_P(PSTR("Low pass")); break;
		case FTHPF: printf_P(PSTR("High pass")); break;
		case FTBPF: printf_P(PSTR("Band pass")); break;
		case FTNOTCH: printf_P(PSTR("Notch")); break;
		case FTAPF: printf_P(PSTR("All pass")); break;
		case FTPEAKING: printf_P(PSTR("Peak.EQ")); break;
		case FTLSHELF: printf_P(PSTR("Low shelf")); break;
		case FTHSHELF: printf_P(PSTR("High shelf")); break;
		default: printf_P(PSTR("f%d"),f->ftype);
		}
		printf_P(PSTR("       "));

		glcdSetAddress(15,LINE4);
		printf_P(PSTR("Freq: %u  "),f->f0);
		glcdSetAddress(15,LINE5);
		if (f->ftype==FTGAIN||f->ftype==FTPEAKING||f->ftype==FTLSHELF||f->ftype==FTHSHELF) {
			printf_P(PSTR("Gain: %d dB  "),(int)f->dbGain);
		} else {
			printf_P(PSTR("                "));
		}
		glcdSetAddress(15,LINE6);
		if (f->ftype!=FTGAIN) {
			unsigned int qp=(int)(f->Q*10);
			printf_P(PSTR("Q: %u.%.1u  "),qp/10,qp%10);
		} else {
			printf_P(PSTR("                "));
		}


		if (GetButtons()&BTNLEFT&&f->ftype>0) {f->ftype--; while (GetButtons()) {DelayMs(10);};};
		if (GetButtons()&BTNRIGHT&&f->ftype<FTHSHELF) {f->ftype++; while (GetButtons()) {DelayMs(10);};};
		if (GetButtons()&BTNDOWN&&f->Q>0.2) {f->Q-=.1999; while (GetButtons()) {DelayMs(10);};};
		if (GetButtons()&BTNUP&&f->Q<10.0) {f->Q+=.1999; while (GetButtons()) {DelayMs(10);};};

		if (fi1|fi2) {
			fdata fd;
			ComputeFilter(f,&fd);
			if (id) {
				TASSendCoef(TASS, fi1, &fd);
			} else {
				TASSendCoef(TASM, fi1, &fd);
			}
			if (fi2) {
				DelayMs(1);
				if (id) {
					TASSendCoef(TASS, fi2, &fd);
				} else {
					TASSendCoef(TASM, fi2, &fd);
				}
			}
		}

	}

}


void SubEQEdit(unsigned char id) {
	//obrazovka pro editaci eq
	unsigned char bqid=0;
	unsigned char showall=0;
	while (GetButtons()) {DelayMs(5);}
	//SetRotary(100, mainmix[id].inpmix[inp].volume);

	while (!(GetButtons()&BTNPWR)) {
		RefreshVolume();
		glcdClearDisp();
		//titulek
		glcdSetAddress(0,LINE1);
		glcdFont(FP_NEGATIVE);
		printf_P(PSTR("    Sub EQ %c      "),id==0 ? 'A' : 'C');
		glcdFont(0);


		//vypis eq

		//vykresleni odezvy

		//zakresleni os (vert. rozmer 9..45, stred 27; horiz. 28..127)
		glcdDottedLine(26,27,127,27);
		glcdDottedLine(26,45,127,45);
		glcdLine(28,9,28,46);
		glcdLine(50,9,50,46);
		glcdDottedLine(61,9,61,46);
		glcdLine(83,9,83,46);
		glcdDottedLine(94,9,94,46);
		glcdLine(116,9,116,46);
		glcdSetAddress(10,LINE2);
		printf_P(PSTR("+18"));
		glcdSetAddress(15,LINE4);
		printf_P(PSTR("0"));
		glcdSetAddress(10,LINE6);
		printf_P(PSTR("-18"));
		//kresleni odezvy
		unsigned char lasty;
		for (unsigned char u=0; u<100 && (!GetButtons()); u++) {
			//kreslime v bode u-1
			//frekvence:
			unsigned int freq=(unsigned int)pow(10,u/33.0+1.3);
			float odezva;
			if (showall) {
				odezva=1.0;
				for (unsigned char i=0; i<16; i++) {
					odezva*=FlAmp(subeq[16*id+i],freq);
				}
			} else {
				odezva=FlAmp(subeq[16*id+bqid],freq);
			}
			unsigned char k=KToDb(odezva);
			if (k>128+18) {k=128+18;};
			if (k<128-18) {k=128-18;};
			k=155-k;
			if (u) {
				glcdSetDot(28+u,k);
			}
			lasty=k;
		}

		//subeq[16*id+bqid]

		glcdSetAddress(0,LINE8);

		printf_P(PSTR("Filter %d"),bqid+1);
		glcdSetAddress(50,LINE8);
		if (!showall) {
			if (subeq[16*id+bqid].ftype==FTGAIN||subeq[16*id+bqid].ftype==FTPEAKING||subeq[16*id+bqid].ftype==FTLSHELF||subeq[16*id+bqid].ftype==FTHSHELF) {
				printf_P(PSTR("%d dB"),(int)subeq[16*id+bqid].dbGain);
			}
			if (subeq[16*id+bqid].ftype!=FTGAIN) {
				glcdSetAddress(85,LINE8);
				printf_P(PSTR("%d Hz"),(int)subeq[16*id+bqid].f0);
			}
		} else {
			glcdFont(FP_NEGATIVE);
			printf_P(PSTR(" Complete char. "));
			glcdFont(0);
		}
		fdata fd;
		ComputeFilter(&subeq[16*id+bqid],&fd);
		
		if (id) {
			TASSendCoef(TASS, (bqid<12)?0x67+bqid:0x48+bqid-12, &fd);
			DelayMs(1);			
		} else {
			TASSendCoef(TASM, (bqid<12)?0x67+bqid:0x48+bqid-12, &fd);
			DelayMs(1);			
		}
		while (!GetButtons()) {RefreshVolume(); DelayMs(5);}
		if (GetButtons()&BTNRIGHT&&bqid<15) {bqid++;}
		if (GetButtons()&BTNLEFT&&bqid) {bqid--;}
		if (GetButtons()&BTNUP&&subeq[16*id+bqid].dbGain<19.99) {subeq[16*id+bqid].dbGain++;}
		if (GetButtons()&BTNDOWN&&subeq[16*id+bqid].dbGain>-79.99) {subeq[16*id+bqid].dbGain--;}
		if (GetButtons()&BTNROT) {showall=!showall;}
		if (GetButtons()&BTNOK) {
			if (bqid<12) {
				BiquadEdit(&subeq[16*id+bqid],0x67+bqid,0,id);
				while (GetButtons()) {DelayMs(5);}
				glcdClearDisp();
			} else {
				BiquadEdit(&subeq[16*id+bqid],0x48+bqid-12,0,id);
				while (GetButtons()) {DelayMs(5);}
				glcdClearDisp();
			}
		}
		while (GetButtons()&~BTNPWR) {DelayMs(5);}
	}
	glcdClearDisp();
	SaveSubEq(id, 1);
	
}


void SysSetup(void) {
	glcdClearDisp();
	//titulek
	glcdSetAddress(0,LINE1);
	glcdFont(FP_NEGATIVE);
	printf_P(PSTR("    System setup     "));
	unsigned char volba=0;
	unsigned char max = 5;
	glcdFont(0);
	glcdSetAddress(5,LINE2);
	printf_P(PSTR("Main volume affects:"));
	
	
	while (!(GetButtons()&(BTNPWR))) {
		
		while (GetButtons()) {DelayMs(10);};
		glcdSetAddress(05,LINE4);
		for (unsigned char i=0; i<=max; i++) {
			if (volset&(1<<i)) {
				glcdFont(FP_ITALIC|(i==volba?FP_NEGATIVE:0));
			} else {
				glcdFont((i==volba?FP_NEGATIVE:0));
			}
			switch (i) {
				case 0:printf_P(PSTR(" A-L"));break;
				case 1:printf_P(PSTR(" A-R"));break;
				case 2:printf_P(PSTR(" B"));break;
				case 3:printf_P(PSTR(" C-L"));break;
				case 4:printf_P(PSTR(" C-R"));break;
				case 5:printf_P(PSTR(" D"));break;
			}
			
		}
		printf_P(PSTR("     "));
		while (!GetButtons()) {RefreshVolume();};
		if (GetButtons()&BTNLEFT&&volba) {volba--;};
		if (GetButtons()&BTNRIGHT&&volba<max) {volba++;};
		if (GetButtons()&BTNOK) {
			volset=volset ^ (1<<volba);
		};
	}
	while (GetButtons()) {DelayMs(10);};	
	eeprom_write_byte((void *)734,volset);
}

void SysInfo(void) {
	glcdClearDisp();
	//titulek
	glcdSetAddress(0,LINE1);
	glcdFont(FP_NEGATIVE);
	printf_P(PSTR("    System info     "));
	
	glcdFont(FP_ITALIC);
	glcdSetAddress(1,LINE3);
	printf_P(PSTR("TLE-3 HiFi Preamp"));
	glcdLine(1,23,110,23);
	glcdFont(0);
	glcdSetAddress(5,LINE4);
	printf_P(PSTR("firmware ver. "VERZEFW));
	glcdSetAddress(5,LINE5);
	printf_P(PSTR("GPL licensed."));
	glcdSetAddress(5,LINE6);
	printf_P(PSTR("(c) Ivo Strasil 2007"));
	glcdSetAddress(45,LINE8);
	printf_P(PSTR("www.strasil.cz"));
	
	
	while (!(GetButtons()&(BTNPWR))) {
		RefreshVolume();
	}
	while (GetButtons()) {DelayMs(10);};	
}



void SetupMenu(void) {
	//zobrazeni menu
	unsigned char menux=0;
	unsigned char menuy=0;

	while (!(GetButtons()&BTNPWR)) {
		RefreshVolume();
		while ((GetButtons())) {DelayMs(30);};
		glcdClearDisp();
		//titulek
		glcdSetAddress(0,LINE1);
		glcdFont(FP_NEGATIVE);
		printf_P(PSTR("    TLE-3 Setup     "));

		glcdFont(FP_BOLD);

		glcdSetAddress(3,LINE3);
		printf_P(PSTR("Mix B"));
		glcdSetAddress(3,LINE4);
		printf_P(PSTR("Mix D"));
		glcdSetAddress(3,LINE5);
		printf_P(PSTR("Eq B"));
		glcdSetAddress(3,LINE6);
		printf_P(PSTR("Eq D"));

		glcdSetAddress(64,LINE3);
		printf_P(PSTR("Out A"));
		glcdSetAddress(64,LINE4);
		printf_P(PSTR("Out C"));
		glcdSetAddress(64,LINE5);
		printf_P(PSTR("Dyn A"));
		glcdSetAddress(64,LINE6);
		printf_P(PSTR("Dyn C"));


		glcdSetAddress(3,LINE7);
		printf_P(PSTR("Dyn B"));
		glcdSetAddress(3,LINE8);
		printf_P(PSTR("Dyn D"));




		if (menux>3&&menuy>3) {menux=3;}


		for (unsigned char i=0; i<6; i++) {
			for (unsigned char j=0; j<6; j++) {

				if (i==menux&&j==menuy) {
					glcdFont(FP_NEGATIVE);
				} else {
					glcdFont(0);
				}
				glcdSetAddress(37+8*(i%3)+62*(i/3),LINE3+j);

				if ((i%3)==0 && (j<4||i<3)) {
					printf_P(PSTR(">"));
				}

				if (j<4||i<3) {

					if ((i%3)==1) {
						printf_P(PSTR(" L"));
					}
					if ((i%3)==2) {
						printf_P(PSTR("S"));
					}

				} else {
					if (i==3) {
						glcdSetAddress(37+62,LINE3+j);
						printf_P(PSTR("edit"));
					}
				}

			}
		}


		glcdSetAddress(64,LINE7);
		printf_P(PSTR("Setup"));
		glcdSetAddress(64,LINE8);
		printf_P(PSTR("Info"));

		//glcdClearDisp();

		while (!(GetButtons())) { RefreshVolume();};

		if ((GetButtons()&BTNRIGHT)&&menux<5) {menux++;}
		if ((GetButtons()&BTNLEFT)&&menux) {menux--;}
		if ((GetButtons()&BTNUP)&&menuy) {menuy--;}
		if ((GetButtons()&BTNDOWN)&&menuy<5) {menuy++;}

		if ((GetButtons()&BTNOK)) {
			while ((GetButtons())) {DelayMs(50);};
			if (menux==0) {
				if (menuy<2) {
					//zobrazime editor hlavniho mixu
					MainMixEdit(menuy);
				} else if (menuy<4) {
					//zobrazime editor EQ
					MainEQEdit(menuy-2);
					//SaveEq(menuy-2,0);
				} else {
					DRCEdit(2*(menuy-4)+1);

				}
			}
			if (menux==1) {
				if (menuy<2) {
					//load hlavniho mixu
					unsigned char p = SelectNumber(1,6);
					if (p)
						LoadMainMix(menuy,p);
				} else if (menuy<4){
					//
					unsigned char p = SelectNumber(1,2);
					if (p)
						LoadEq(menuy-2,p-1);
				} else {
					unsigned char p = SelectNumber(1,4);
					if (p) {LoadDRCFromEE(2*(menuy-4)+1,p);};
				}
			}
			if (menux==2) {
				if (menuy<2) {
					//save hlavniho mixu
					unsigned char p = SelectNumber(1,6);
					if (p)
						SaveMainMix(menuy,p);
				} else if (menuy<4) {
					//
					unsigned char p = SelectNumber(1,2);
					if (p)
						SaveEq(menuy-2,p-1);
				} else {
					unsigned char p = SelectNumber(1,4);
					if (p) {SaveDRCToEE(2*(menuy-4)+1,p);};
				}
			}

			if (menux==3) {
				if (menuy<2) {
					//zobrazime editor hlavniho mixu
					SubMixEdit(menuy);
				} else if (menuy<4) {
					//zobrazime editor EQ
					DRCEdit(2*(menuy-2));

				} else {
					//setup, info
					if (menuy==4) {SysSetup();}
					if (menuy==5) {SysInfo();}
				}
			}
			if (menux==4) {
				if (menuy<2) {
					//load sub mixu
					unsigned char p = SelectNumber(1,2);
					if (p)
						LoadSubMixFromEE(menuy,p);
				} else if (menuy<4){
					//drc subu
					unsigned char p = SelectNumber(1,4);
					if (p) {LoadDRCFromEE(2*(menuy-2),p);};
				} else {

				}
			}
			if (menux==5) {
				if (menuy<2) {
					//save sub mixu
					unsigned char p = SelectNumber(1,2);
					if (p)
						SaveSubMixToEE(menuy,p);
				} else if (menuy<4) {
					//save drc subu
					unsigned char p = SelectNumber(1,4);
					if (p) {SaveDRCToEE(2*(menuy-2),p);};
				} else {

				}
			}




			while ((GetButtons())) {DelayMs(50);};
		}



	}
}

void MainEQEdit(unsigned char id) {
	//obrazovka pro editaci eq
	unsigned char bqid=0;
	unsigned char showall=0;

	//SetRotary(100, mainmix[id].inpmix[inp].volume);

	while (!(GetButtons()&BTNPWR)) {
		RefreshVolume();
		glcdClearDisp();
		//titulek
		glcdSetAddress(0,LINE1);
		glcdFont(FP_NEGATIVE);
		printf_P(PSTR("    Main EQ %c      "),id==0 ? 'B' : 'D');
		glcdFont(0);


		//vypis eq

		//vykresleni odezvy

		//zakresleni os (vert. rozmer 9..45, stred 27; horiz. 28..127)
		glcdDottedLine(26,27,127,27);
		glcdDottedLine(26,45,127,45);
		glcdLine(28,9,28,46);
		glcdLine(50,9,50,46);
		glcdDottedLine(61,9,61,46);
		glcdLine(83,9,83,46);
		glcdDottedLine(94,9,94,46);
		glcdLine(116,9,116,46);
		glcdSetAddress(10,LINE2);
		printf_P(PSTR("+18"));
		glcdSetAddress(15,LINE4);
		printf_P(PSTR("0"));
		glcdSetAddress(10,LINE6);
		printf_P(PSTR("-18"));
		//kresleni odezvy
		unsigned char lasty;
		for (unsigned char u=0; u<100&&!GetButtons(); u++) {
			//kreslime v bode u-1
			//frekvence:
			unsigned int freq=(unsigned int)pow(10,u/33.0+1.3);
			float odezva;
			if (showall) {
				odezva=1.0;
				for (unsigned char i=0; i<12; i++) {
					odezva*=FlAmp(maineq[12*id+i],freq);
				}
			} else {
				odezva=FlAmp(maineq[12*id+bqid],freq);
			}
			unsigned char k=KToDb(odezva);
			if (k>128+18) {k=128+18;};
			if (k<128-18) {k=128-18;};
			k=155-k;
			if (u) {
				glcdSetDot(28+u,k);
			}
			lasty=k;
		}

		//maineq[12*id+bqid]

		glcdSetAddress(0,LINE8);

		printf_P(PSTR("Filter %d"),bqid+1);
		glcdSetAddress(50,LINE8);
		if (!showall) {
			if (maineq[12*id+bqid].ftype==FTGAIN||maineq[12*id+bqid].ftype==FTPEAKING||maineq[12*id+bqid].ftype==FTLSHELF||maineq[12*id+bqid].ftype==FTHSHELF) {
				printf_P(PSTR("%d dB"),(int)maineq[12*id+bqid].dbGain);
			}
			if (maineq[12*id+bqid].ftype!=FTGAIN) {
				glcdSetAddress(85,LINE8);
				printf_P(PSTR("%d Hz"),(int)maineq[12*id+bqid].f0);
			}
		} else {
			glcdFont(FP_NEGATIVE);
			printf_P(PSTR(" Complete char. "));
			glcdFont(0);
		}
		fdata fd;
		ComputeFilter(&maineq[12*id+bqid],&fd);
		if (id) {
			TASSendCoef(TASS, 0x4F+bqid, &fd);
			DelayMs(1);
			TASSendCoef(TASS, 0x5B+bqid, &fd);
		} else {
			TASSendCoef(TASM, 0x4F+bqid, &fd);
			DelayMs(1);
			TASSendCoef(TASM, 0x5B+bqid, &fd);
		}
		while (!GetButtons()) {RefreshVolume(); DelayMs(5);}
		if (GetButtons()&BTNRIGHT&&bqid<11) {bqid++;}
		if (GetButtons()&BTNLEFT&&bqid) {bqid--;}
		if (GetButtons()&BTNUP&&maineq[12*id+bqid].dbGain<19.99) {maineq[12*id+bqid].dbGain++;}
		if (GetButtons()&BTNDOWN&&maineq[12*id+bqid].dbGain>-79.99) {maineq[12*id+bqid].dbGain--;}
		if (GetButtons()&BTNROT) {showall=!showall;}
		if (GetButtons()&BTNOK) {BiquadEdit(&maineq[12*id+bqid],0x4F+bqid,0x5B+bqid,id);}
		while (GetButtons()&~BTNPWR) {DelayMs(5);}
	}

}





void MainMixEdit(unsigned char id) {
	//obrazovka pro editaci mixu
	//edituji mainmix[id]
	glcdClearDisp();
	//titulek
	glcdSetAddress(0,LINE1);
	glcdFont(FP_NEGATIVE);
	printf_P(PSTR("    Output %c mix     "),id==0 ? 'B' : 'D');

	unsigned char inp=0;
	unsigned char param=0;
	unsigned char paredit=0;

	SetRotary(100, mainmix[id].inpmix[inp].volume);

	while (!(GetButtons()&BTNPWR)) {
		RefreshVolume();
		TASSetupMainMix(id) ;

		//vstup od tlacitek



		if ((GetButtons()&BTNLEFT)&&inp) {
			inp--;
			SetRotary(100, mainmix[id].inpmix[inp].volume);
		}


		if ((GetButtons()&BTNRIGHT)&&inp<3) {
			inp++;
			SetRotary(100, mainmix[id].inpmix[inp].volume);
		}
		if ((GetButtons()&BTNUP)&&param&&!paredit) {
			param--;
		}
		if ((GetButtons()&BTNDOWN)&&((inp==2) ? param<4 : param<3)&&!paredit) {
			param++;
		}
		if ((GetButtons()&BTNOK)&&param!=4) {
			paredit=!paredit;
		}
		if (param==4&&inp!=2) {param=3;}
		if (paredit||param==4) {
			if (param==0) {
				if (GetButtons()&BTNDOWN&&mainmix[id].inpmix[inp].balance>64) {mainmix[id].inpmix[inp].balance--;}
				if (GetButtons()&BTNUP&&mainmix[id].inpmix[inp].balance<192) {mainmix[id].inpmix[inp].balance++;}
			}
			if (param==1) {
				if (GetButtons()&BTNUP) {mainmix[id].inpmix[inp].flags|=CHANNEL_SWAP;}
				if (GetButtons()&BTNDOWN) {mainmix[id].inpmix[inp].flags&=~CHANNEL_SWAP;}
			}
			if (param==2) {
				if (GetButtons()&BTNUP) {mainmix[id].inpmix[inp].flags|=REZ_MONO;}
				if (GetButtons()&BTNDOWN) {mainmix[id].inpmix[inp].flags&=~REZ_MONO;}
			}
			if (param==3) {
				if (GetButtons()&BTNUP) {
					if (mainmix[id].inpmix[inp].flags&PH_INV_R) {
						mainmix[id].inpmix[inp].flags&=~(PH_INV_R);
						mainmix[id].inpmix[inp].flags|=PH_INV_L;
					} else {
						mainmix[id].inpmix[inp].flags&=~(PH_INV_L);
						mainmix[id].inpmix[inp].flags|=PH_INV_R;
					}
				}

				if (GetButtons()&BTNDOWN) {mainmix[id].inpmix[inp].flags&=~(PH_INV_L|PH_INV_R);}
			}
			if (param==4&&(GetButtons()&BTNOK)) {
				if (mainmix[id].inpmix[inp].flags&REZ_EXTCHAN) {
					mainmix[id].inpmix[inp].flags&=~(REZ_EXTCHAN);
				} else {
					mainmix[id].inpmix[inp].flags|=REZ_EXTCHAN;
				}
			}
		}

		//prenos extchan naproti
		mainmix[1-id].inpmix[inp].flags&=~REZ_EXTCHAN;
		if (mainmix[id].inpmix[inp].flags&REZ_EXTCHAN) {
			mainmix[1-id].inpmix[inp].flags|=REZ_EXTCHAN;
		}

		while (GetButtons()&(~BTNPWR)&(~BTNROT)) {};

		CSSetIn(mainmix[id].inpmix[2].flags&REZ_EXTCHAN);


		//vypis bargrafu vstupu
		for (unsigned char i=0; i<4; i++) {
			//nadpisy
			if (inp==i&&!paredit) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
			glcdSetAddress(i*32+5,LINE2);
			if (i<3) {
				if ((i!=2)||(!(mainmix[id].inpmix[i].flags&REZ_EXTCHAN))) {
					printf_P(PSTR(" INP%d "),i+1);
				} else {
					printf_P(PSTR(" INP4 "));
				}
			} else {
				printf_P(PSTR(" USB "),i);
			}
			//bargraf urovne
			if (inp==i) {
				glcdClrRectangle(inp*32+2, 20+((100-mainmix[id].inpmix[inp].volume)/3), 3, 3);
				if ((GetButtons()&BTNROT)) {
					if (GetRotary()==80) {
						SetRotary(100, 0);
					} else {
						SetRotary(100, 80);
					}

				}
				while (GetButtons()&(BTNROT)) {};
				mainmix[id].inpmix[inp].volume=GetRotary();
			}
			glcdRectangle(i*32+2, 20+((100-mainmix[id].inpmix[i].volume)/3), 3,3);
			//vypis parametru
			//hlasitost
			glcdSetAddress(i*32+3+3,LINE3);
			glcdFont(FP_ITALIC);
			if (mainmix[id].inpmix[i].volume) {
				printf_P(PSTR("%+2d  "),(signed char)mainmix[id].inpmix[i].volume-80);
			} else {
				printf_P(PSTR("MUTE   "));
			}
			//balance
			glcdSetAddress(i*32+3+3,LINE4);
			if (inp==i&&param==0) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
			if (mainmix[id].inpmix[i].balance==128) {
				printf_P(PSTR(">0< "));
			} else if (mainmix[id].inpmix[i].balance>128) {
				printf_P(PSTR("<-%d "),mainmix[id].inpmix[i].balance-128);
			} else {
				printf_P(PSTR("-%d> "),128-mainmix[id].inpmix[i].balance);
			}
			//rezimy
			glcdSetAddress(i*32+3+3,LINE5);
			if (inp==i&&param==1) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
			if (mainmix[id].inpmix[i].flags&CHANNEL_SWAP) {
				if (mainmix[id].inpmix[i].flags&REZ_MONO) {
					printf_P(PSTR("Karao"));
				}
				else  {
					printf_P(PSTR("SWP  "));
				}

			} else {
				printf_P(PSTR("NOR  "));
			}
			glcdSetAddress(i*32+3+3,LINE6);
			if (inp==i&&param==2) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
			if (mainmix[id].inpmix[i].flags&REZ_MONO) {
				printf_P(PSTR("MONO "));
			} else {
				printf_P(PSTR("STER "));
			}
			glcdSetAddress(i*32+3+3,LINE7);
			if (inp==i&&param==3) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
			if (mainmix[id].inpmix[i].flags&PH_INV_L) {
				printf_P(PSTR("IvL "));
			} else if (mainmix[id].inpmix[i].flags&PH_INV_R) {
				printf_P(PSTR("IvR "));
			} else {
				printf_P(PSTR("NoIv "));
			}

			if (i==2) {
				glcdSetAddress(i*32+3+3,LINE8);
				if (inp==i&&param==4) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
				printf_P(PSTR("Channel"));
			}




		}
		DelayMs(10);
	}



}






void SubMixEdit(unsigned char id) {
	//obrazovka pro editaci mixu
	//edituji submix[id]
	glcdClearDisp();
	//titulek


	unsigned char inp=0;
	
	SetRotary(100, submix[id].gain[inp]);

	while (!(GetButtons()&BTNPWR)) {
		RefreshVolume();





		// TASSetupSubmix(id) ;




		//vstup od tlacitek
		if ((GetButtons()&BTNOK)) {
			SubEQEdit(id);
		}
		if ((GetButtons()&BTNLEFT)&&inp) {
			inp--;
			if ((inp%2==1) && submix[id].monoset[inp/2]) {
				inp--;
			}
			SetRotary(100, submix[id].gain[inp]);
			if (inp==9) {glcdClearDisp();}
		}
		if ((GetButtons()&BTNRIGHT)&&inp<16) {
			if ((inp%2==0) && submix[id].monoset[inp/2]) {
				inp++;
			}
			inp++;
			SetRotary(100, submix[id].gain[inp]);
			if (inp==10) {glcdClearDisp();}
		}

		/*if ((GetButtons()&BTNOK)) {
		   paredit=!paredit;
		   }*/


		if ((GetButtons()&BTNUP)) {
			if (submix[id].monoset[inp/2]) {
				submix[id].monoset[inp/2]=0;
			} else {
				submix[id].monoset[inp/2]=1;
			}
		}

		submix[id].monoset[6]=1;
		submix[id].monoset[8]=1;

		if (submix[id].monoset[inp/2]&&(inp%2)) {
			inp--;
		}


		if ((GetButtons()&BTNDOWN)&&inp<10) {
			if (submix[id].invflags&(1<<inp)) {
				submix[id].invflags&=~(1<<inp);
			} else {
				submix[id].invflags|=(1<<inp);
			}
		}
		while (GetButtons()&(~BTNPWR)&(~BTNROT)) {};

		//vypis bargrafu vstupu

		//volba obrazovky

		if (inp<10) {

			for (unsigned char i=0; i<10; i++) {
				//nadpisy
				if (i%2==0)  {
					if (inp/2==i/2) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
					glcdSetAddress((i/2)*25+5,LINE2);
					if ((i/2)<3) {
						if (((i/2)!=2)||(!(mainmix[0].inpmix[2].flags&REZ_EXTCHAN))) {
							printf_P(PSTR(" %d "),i/2+1);
						} else {
							printf_P(PSTR(" 4 "));
						}
					} else if ((i/2)==3) {
						printf_P(PSTR("USB"));
					} else {
						printf_P(PSTR("Main"));
					}
				}

				//bargraf urovne
				if (inp==i) {
					for (unsigned char j=1; j<4; j++) {
						glcdClearDot  ((i/2)*25+10+4+j+2-1, 26+1+((100-submix[id].gain[i])/5));
					}

					if ((GetButtons()&BTNROT)) {
						if (submix[id].gain[inp]==80) {
							SetRotary(100, 0);
						} else {
							SetRotary(100, 80);
						}
						while (GetButtons()&(BTNROT)) {};
					}

					if (submix[id].monoset[inp/2]) {
						glcdClrRectangle((i/2)*25+10+2, 26+((100-submix[id].gain[(inp/2)*2])/5), 3, 3);
						glcdClrRectangle((i/2)*25+10+8+2, 26+((100-submix[id].gain[(inp/2)*2+1])/5), 3, 3);
						submix[id].gain[(inp/2)*2]=GetRotary();
						submix[id].gain[(inp/2)*2+1]=GetRotary();
						glcdRectangle(((i)/2)*25+10+2, 26+((100-submix[id].gain[(i)])/5), 3,3);
						glcdRectangle(((i)/2)*25+10+2+8, 26+((100-submix[id].gain[(i)])/5), 3,3);
					} else {
						glcdClrRectangle((i/2)*25+10+(i%2)*8+2, 26+((100-submix[id].gain[inp])/5), 3, 3);
						submix[id].gain[inp]=GetRotary();
					}


				}

				glcdRectangle((i/2)*25+10+(i%2)*8+2, 26+((100-submix[id].gain[i])/5), 3,3);

				if ((i%2)==0) {

					for (unsigned char j=1; j<4; j++) {
						if (submix[id].monoset[i/2]) {
							glcdSetDot  ((i/2)*25+10+4+j+2-1, 26+1+((100-submix[id].gain[i])/5));
						} else {
							glcdClearDot  ((i/2)*25+10+4+j+2-1, 26+1+((100-submix[id].gain[i])/5));
						}
					}

				}

				glcdSetDot  ((i/2)*25+10+(i%2)*8+2-1, 26+1+((100-80)/5));
				glcdSetDot  ((i/2)*25+10+(i%2)*8+2+3, 26+1+((100-80)/5));

				//vypis parametru
				//hlasitost

				glcdSetAddress((i/2)*25+10+(i%2)*8+3,LINE3);
				glcdFont(0);
				if (((unsigned short)(submix[id].invflags)>>i)&0x0001) {
					printf_P(PSTR("i "));
				} else {
					if (i%2){
						printf_P(PSTR("R"));
					} else {
						printf_P(PSTR("L"));
					}
				}




			}
		} else {
			//zbylych 6 kanalu - nastaveni mixu
			for (unsigned char i=10; i<18; i++) {
				//nadpisy
				if (i%2==0)  {
					glcdFont(0);
					glcdSetAddress(((i-10)/2)*25+18,LINE2);
					if (i==10) {
						printf_P(PSTR("Left ch."));
					}
					if (i==14) {
						printf_P(PSTR("Right ch."));
					}
				}

				//bargraf urovne
				if (inp==i) {
					for (unsigned char j=1; j<4; j++) {
						glcdClearDot  ((i-10)/2*25+10+4+j+2-1, 26+1+((100-submix[id].gain[i])/5));
					}


					if ((GetButtons()&BTNROT)) {
						if (submix[id].gain[inp]==80) {
							SetRotary(100, 0);
						} else {
							SetRotary(100, 80);
						}
						while (GetButtons()&(BTNROT)) {};
					}

					if (submix[id].monoset[(inp)/2]) {
						glcdClrRectangle(((i-10)/2)*25+10+2, 26+((100-submix[id].gain[((inp)/2)*2])/5), 3, 3);
						glcdClrRectangle(((i-10)/2)*25+10+8+2, 26+((100-submix[id].gain[((inp)/2)*2+1])/5), 3, 3);
						submix[id].gain[((inp)/2)*2]=GetRotary();
						submix[id].gain[((inp)/2)*2+1]=GetRotary();
						glcdRectangle(((i-10)/2)*25+10+2, 26+((100-submix[id].gain[(i)])/5), 3,3);
						glcdRectangle(((i-10)/2)*25+10+2+8, 26+((100-submix[id].gain[(i)])/5), 3,3);
					} else {
						glcdClrRectangle(((i-10)/2)*25+10+((i-10)%2)*8+2, 26+((100-submix[id].gain[(inp)])/5), 3, 3);
						submix[id].gain[(inp)]=GetRotary();
					}


				}

				glcdRectangle(((i-10)/2)*25+10+((i-10)%2)*8+2, 26+((100-submix[id].gain[(i)])/5), 3,3);

				if (((i-10)%2)==0) {

					for (unsigned char j=1; j<4; j++) {
						if (submix[id].monoset[(i)/2]) {
							glcdSetDot  (((i-10)/2)*25+10+4+j+2-1, 26+1+((100-submix[id].gain[(i)])/5));
						} else {
							glcdClearDot  (((i-10)/2)*25+10+4+j+2-1, 26+1+((100-submix[id].gain[(i)])/5));
						}
					}

				}

				glcdSetDot  (((i-10)/2)*25+10+((i-10)%2)*8+2-1, 26+1+((100-80)/5));
				glcdSetDot  (((i-10)/2)*25+10+((i-10)%2)*8+2+3, 26+1+((100-80)/5));

				//vypis parametru
				//hlasitost

				glcdSetAddress(((i-10)/2)*25+10+((i-10)%2)*8+3,LINE3);
				glcdFont(0);
				if (inp==i) {
					glcdFont(FP_NEGATIVE);
				}
				if (((unsigned short)(submix[id].invflags)>>(i-10))&0x0001) {
					printf_P(PSTR("i"));
				} else {
					if ((i/2)==6||(i/2)==8) {
						if (!((i-10)%2)){
							printf_P(PSTR("mono"));
						}
					} else {
						if (i%2){
							printf_P(PSTR("R"));
						} else {
							printf_P(PSTR("L"));
						}
						if (((unsigned short)(submix[id].invflags)>>i)&0x0001) {
							printf_P(PSTR("i"));
						} else {
							printf_P(PSTR(" "));
						}
					}
				}




			}

		}

		//volba kanalu - vypis uplne dole, L/R/mono
		glcdFont(0);
		glcdSetAddress(10,LINE8);
		if (inp<10) {
			printf_P(PSTR("Inp. "));
		} else {
			printf_P(PSTR("Out "));
		}
		if ((inp/2)<3) {
			if (((inp/2)!=2)||(!(mainmix[0].inpmix[2].flags&REZ_EXTCHAN))) {
				printf_P(PSTR(" %d "),inp/2+1);
			} else {
				printf_P(PSTR(" 4"));
			}
		} else if ((inp/2)==3) {
			printf_P(PSTR("USB"));
		} else if ((inp/2)==4) {
			printf_P(PSTR("m.mix"));
		} else {
			if (inp<14) {
				printf_P(PSTR("L"));
			} else {
				printf_P(PSTR("R"));
			}
		}
		printf_P(PSTR(", "));
		if (submix[id].monoset[inp/2]) {
			printf_P(PSTR("L-R"));
		} else {
			if (inp%2) {
				printf_P(PSTR(" R"));
			} else {
				printf_P(PSTR(" L"));
			}
		}
		printf_P(PSTR(": "));
		if (submix[id].gain[inp]) {
			printf_P(PSTR("%+2d dB        "),(signed char)submix[id].gain[inp]-80);
		} else {
			printf_P(PSTR("MUTE        "));
		}

		glcdSetAddress(20,LINE1);
		glcdFont(FP_NEGATIVE);
		if (inp<10) {
			printf_P(PSTR("  Output %c monomix   "),id==0 ? 'A' : 'C');
		} else {
			printf_P(PSTR("  Output %c out mix   "),id==0 ? 'A' : 'C');
		}

		LoadSubMix(id);
		DelayMs(7);
	}



}




void DRCEdit(unsigned char id) {
	//obrazovka pro editaci mixu
	//edituji mainmix[id]
	glcdClearDisp();
	//titulek
	glcdSetAddress(0,LINE1);
	glcdFont(FP_NEGATIVE);
	printf_P(PSTR(" %c out dynamic processor "),id+'A');

	unsigned char inp=0;
	unsigned char param=0;
	unsigned char paredit=0;

	SetRotary(100, mainmix[id].inpmix[inp].volume);

	while (!(GetButtons()&BTNPWR)) {
		RefreshVolume();

		//vstup od tlacitek
		if ((GetButtons()&BTNLEFT)&&inp) {
			inp--;
			if (((inp==1)||(inp==2))&&param<2) {
				if (param==0) {
					SetRotary(80, drcset[id].treshold[inp-1]);
				}
				if (param==1) {
					SetRotary(100, drcset[id].offset[inp-1]);
				}
			}
		}
		if ((GetButtons()&BTNRIGHT)&&inp<3) {
			inp++;
			if (((inp==1)||(inp==2))&&param<2) {
				if (param==0) {
					SetRotary(80, drcset[id].treshold[inp-1]);
				}
				if (param==1) {
					SetRotary(100, drcset[id].offset[inp-1]);
				}
			}
		}
		if ((GetButtons()&BTNUP)&&param&&!paredit) {
			param--;
			if (((inp==1)||(inp==2))&&param<2) {
				if (param==0) {
					SetRotary(80, drcset[id].treshold[inp-1]);
				}
				if (param==1) {
					SetRotary(100, drcset[id].offset[inp-1]);
				}
			}
		}
		if ((GetButtons()&BTNDOWN)&&param<1&&!paredit) {
			param++;
			if (((inp==1)||(inp==2))&&param<2) {
				if (param==0) {
					SetRotary(80, drcset[id].treshold[inp-1]);
				}
				if (param==1) {
					SetRotary(100, drcset[id].offset[inp-1]);
				}
			}
		}
		if ((GetButtons()&BTNOK)) {
			paredit=!paredit;
			if (((inp==1)||(inp==2))&&param<2) {
				if (param==0) {
					SetRotary(80, drcset[id].treshold[inp-1]);
				}
				if (param==1) {
					SetRotary(100, drcset[id].offset[inp-1]);
				}
			}

		}

		if (!paredit && ((inp==1)||(inp==2)))  {
			if (param==0) {
				drcset[id].treshold[inp-1] = GetRotary();
			}
			if (param==1) {
				drcset[id].offset[inp-1] = GetRotary();
			}
		}

/*
   typedef struct {
   unsigned char slopeL, slopeH;
   unsigned char treshold[2];
   unsigned char offset[2];
   unsigned char speed;
   } t_drc_struct ;
 */

		{
			if (inp==0) {
				if (GetButtons()&BTNDOWN&&drcset[id].slopeL) {drcset[id].slopeL--;}
				if (GetButtons()&BTNUP&&drcset[id].slopeL<100) {drcset[id].slopeL++;}
			} else if (inp==3) {
				if (GetButtons()&BTNDOWN&&drcset[id].slopeH) {drcset[id].slopeH--;}
				if (GetButtons()&BTNUP&&drcset[id].slopeH<100) {drcset[id].slopeH++;}
			} else if (paredit) {
				//ostatni parametry
				if (param==0) {
					if (GetButtons()&BTNDOWN&&drcset[id].treshold[inp-1]) {drcset[id].treshold[inp-1]--;}
					if (GetButtons()&BTNUP&&drcset[id].treshold[inp-1]<80) {drcset[id].treshold[inp-1]++;}
				}
				if (param==1) {
					if (GetButtons()&BTNDOWN&&drcset[id].offset[inp-1]) {drcset[id].offset[inp-1]--;}
					if (GetButtons()&BTNUP&&drcset[id].offset[inp-1]<100) {drcset[id].offset[inp-1]++;}
				}
			}
		}




		while (GetButtons()&(~BTNPWR)) {};


		if (drcset[id].treshold[1]<=drcset[id].treshold[0]) {if (!drcset[id].treshold[1]==80) {drcset[id].treshold[0]=79;}; drcset[id].treshold[1]=drcset[id].treshold[0]+1;};


		//vypis
		for (unsigned char i=0; i<4; i++) {
			//nadpisy
			if (inp==i&&!paredit) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
			glcdSetAddress(i*24+32,LINE2);
			if (i==1||i==2) {
				printf_P(PSTR(" T%d "),i);
			} else {
				if (!i) {
					printf_P(PSTR(" Low "));
				} else {
					printf_P(PSTR(" High"));
				}
			}

			//vypis parametru

			//slope (MEZI sloupci)

			if (i<3) {

				glcdSetAddress(i*32+32,LINE3);

				glcdFont(FP_ITALIC);

				if (i==0) {
					printf_P(PSTR("%1d,%.1d:1 "),drcset[id].slopeL/10,drcset[id].slopeL%10);
				} else if (i==1) {
					unsigned short computed_slope = 10*(((unsigned short)drcset[id].treshold[1]+(unsigned short)drcset[id].offset[1]-50-((unsigned short)drcset[id].treshold[0]+(unsigned short)drcset[id].offset[0]-50)))/((unsigned short)drcset[id].treshold[1]-(unsigned short)drcset[id].treshold[0]);
					printf_P(PSTR("%1d,%.1d:1 "),computed_slope/10,computed_slope%10);
				} else {
					printf_P(PSTR("%1d,%.1d:1 "),drcset[id].slopeH/10,drcset[id].slopeH%10);
				}
			}



			if (i==1||i==2) {
				//treshold
				glcdFont(0);
				glcdSetAddress(3,LINE6);
				printf_P(PSTR("Treshold:"));
				glcdSetAddress(3,LINE7);
				printf_P(PSTR("Offset:"));

				glcdSetAddress(i*24+32,LINE6);

				if (inp==i&&param==0) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
				printf_P(PSTR("%+2d"),(signed short)drcset[id].treshold[i-1]-80);
				glcdFont(0);
				printf_P(PSTR("    "));



				glcdSetAddress(i*24+32,LINE7);

				if (inp==i&&param==1) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
				printf_P(PSTR("%+2d"),(signed short)drcset[id].offset[i-1]-50);
				glcdFont(0);
				printf_P(PSTR("    "));


			} else {
				glcdSetAddress(i*23+32,LINE5);
				if (inp==i) {glcdFont(FP_NEGATIVE);} else {glcdFont(0);}
				printf_P(PSTR("Slope"));
			}
		}
		DelayMs(10);

		//nacteni do DRC
		LoadDRC(id);



	}



}




