//*****************************************************************************
//
// Title		: TLE-3 firmware
// Authors		: Ivo Strasil
// File Name	: 'main.c'
// Date			: April 26, 2006
// Version		: 1.00
// Target MCU	: atmega64
// Editor Tabs	: 4

//
//*****************************************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#include <string.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "ks0108.h"
#include "glcd.h"

#include "i2c.h"
#include "hwdef.h"
#include "intf.h"



void mcuInit            (void);
void dir_serial         (unsigned long cluster);
void print_hd_info      (void);
int  uart_putc          (char c);
int  uart_getc          (void);

unsigned char buf[700];
unsigned int wpos,rpos;
volatile signed int delta;
FILE * lcd, usart;

void outdata(unsigned short dataleft, unsigned short dataright);

SIGNAL(SIG_OVERFLOW0)
{
	ProcessRotary();
}








//*****************************************************************************
// Function: main
// Parameters: none
// Returns: 0
//*****************************************************************************
char pdata[0x10];
int main(void)
{

	Fs=96000;
	unsigned char i,j;
	mcuInit();                                                      // start ATMega128
	InitPorts();

	//init cteni rot. enk.
	// Timer/Counter 0 initialization
	// Clock source: System Clock
	// Clock value: 2000,000 kHz
	// Mode: Normal top=FFh
	// OC0 output: Disconnected
	ASSR=0x00;
	TCCR0=0x02;

	TIMSK=0x01;
	ETIMSK=0x00;

	asm ("sei");

	//inicializace I2C
	i2cInit();
	glcdInit();
	lcd=fdevopen( /*glcdWriteChar*/ glcdWriteCharGr, NULL);
	//	usart=fdevopen(uart_putc, uart_getc);	// configure printf to send data to serial port






	printf_P(PSTR("AP init"));
	glcdSetAddress(4,LINE2);

	do {

		volset=eeprom_read_byte((void *)734);

		LCDBackOn();
		ResetLed(0x09);
		SetLed(0x01);

		TASInit();
		RefreshVolume();

		DelayMs(20);

		fprintf_P(lcd,PSTR("I2C comm test:"));

		for (unsigned char i=0x01; i<127; i++) {
			if (i2cMasterSendNI(i*2, 0, pdata)==I2C_OK) {
				fprintf_P(lcd,PSTR("id %3d  "),i*2);
			}
		}

		LoadMainMix(0,1);
		LoadMainMix(1,2);

		//nacteni biquadu spektraku

		for (unsigned char i=0; i<10; i++) {
			unsigned short fkrit[10]={60,170,310,600,1000,3000,6000,12000,15000,18000};
			fdata fd;
			filter fsetup;
			fsetup.ftype = FTBPF;
			fsetup.f0 = fkrit[i];
			fsetup.dbGain = 1.0;
			fsetup.Q = 1.0;
			ComputeFilter(&fsetup,&fd);
			TASSendCoef(TASM, 0xBC+i, &fd);
			DelayUs(200);
		}
		{
			unsigned char lddata[9]={0xBB,0x00,0x19,0x99,0x99,0x00,0x66,0x66,0x66};
			//unsigned char lddata[9]={0xBB,0x00,0x00,0x03,0x46,0x00,0x7F,0xFC,0xB9};
			i2cMasterSendNI(TASM, 9, lddata);
		}


		DelayMs(50);

		/*
		   mainmix[1].inpmix[0].volume=80;
		   mainmix[1].inpmix[1].volume=80;
		   mainmix[1].inpmix[2].volume=80;
		   mainmix[1].inpmix[3].volume=80;

		   mainmix[1].inpmix[0].balance=128;
		   mainmix[1].inpmix[1].balance=128;
		   mainmix[1].inpmix[2].balance=128;
		   mainmix[1].inpmix[3].balance=135;

		   mainmix[1].inpmix[0].flags=REZ_MONO;
		   mainmix[1].inpmix[1].flags=PH_INV_L;
		   mainmix[1].inpmix[2].flags=0;
		   mainmix[1].inpmix[3].flags=CHANNEL_SWAP;*/

		TASSetupMainMix(0) ; TASSetupMainMix(1) ;

		LoadEq(0,0);
		LoadEq(1,0);

		LoadDRCFromEE(0,1);
		LoadDRCFromEE(1,1);
		LoadDRCFromEE(2,1);
		LoadDRCFromEE(3,1);

		LoadSubMixFromEE(0, 1);
		LoadSubMixFromEE(1, 1);

		InitDRC(TASM);
		InitDRC(TASS);
		
		LoadSubEq(0, 1);
		LoadSubEq(1, 1);
		
		PCMInit();

		typedef struct {
			unsigned char slopeL, slopeH;
			unsigned char treshold[2];
			unsigned char offset[2];
			unsigned char speed;
		} t_drc_struct ;
/*
		drcset[0].slopeL=10;
		drcset[0].slopeH=10;
		drcset[0].treshold[0]=40;
		drcset[0].offset[0]=50;
		drcset[0].treshold[1]=10;
		drcset[0].offset[1]=50;
		drcset[0].speed=0;
*/

		for (unsigned char i=0; i<32; i++) {
			if (!subeq[i].f0 || subeq[i].ftype>10)  {
				subeq[i].f0=1000;
				subeq[i].ftype=2;
				subeq[i].dbGain=0;
				subeq[i].Q=0.7;
			}
		}






		glcdClearDisp();

		signed short dbmax[10]={0,0,0,0,0,0,0,0,0,0};
		signed short dbavg2[10]={0,0,0,0,0,0,0,0,0,0};

		unsigned char redr=1;

		while(!(GetButtons()&BTNPWR)) {

			if (GetButtons()&BTNOK) {
				//nastaveni
				SetupMenu();
				redr=1;

				glcdClearDisp();

				while (GetButtons()) {DelayMs(50);}
			}
			
			if (GetButtons()&BTNRIGHT) {
				PCMSendHID(8);
				while (GetButtons()) {DelayMs(50);}
			}
			if (GetButtons()&BTNLEFT) {
				PCMSendHID(16);
				while (GetButtons()) {DelayMs(50);}
			}
			if (GetButtons()&BTNUP) {
				PCMSendHID(64);
				while (GetButtons()) {DelayMs(50);}
			}
			if (GetButtons()&BTNDOWN) {
				PCMSendHID(32);
				while (GetButtons()) {DelayMs(50);}
			}


			RefreshVolume();

			{ /* analyzator */
				if (redr) {
				redr=0;

				glcdSetAddress(0,LINE1);
				glcdFont(FP_NEGATIVE);
				fprintf_P(lcd,PSTR("      TLE-3    Preamp      "));

				//zobrazeni zvolenych vstupu
				for (unsigned char id=0; id<2; id++) {

					glcdSetAddress(0,LINE5+id);
					glcdFont(0);

					if (id) {
						fprintf_P(lcd,PSTR("D:"));
					} else {                       fprintf_P(lcd,PSTR("B:"));}

					for (unsigned char i=0; i<4; i++) {

						if (mainmix[id].inpmix[i].volume>20) {
							glcdFont(FP_NEGATIVE|((mainmix[id].inpmix[i].flags&(REZ_MONO|CHANNEL_SWAP|PH_INV_L|PH_INV_R)) ? FP_ITALIC : 0));
						} else {
							glcdFont(((mainmix[id].inpmix[i].flags&(REZ_MONO|CHANNEL_SWAP|PH_INV_L|PH_INV_R)) ? FP_ITALIC : 0));
						}

						if (i<3) {
							if (i==2&&((mainmix[id].inpmix[i].flags&REZ_EXTCHAN))) {
								fprintf_P(lcd,PSTR(" 4 "));
							} else {
								fprintf_P(lcd,PSTR(" %d "),i+1);
							}
						} else {
							fprintf_P(lcd,PSTR(" U "));
						}
					}
					glcdFont(0);
					fprintf_P(lcd,PSTR(" "));
				}
				
				
				//zobrazeni zvolenych vstupu
				for (unsigned char id=0; id<2; id++) {

					glcdSetAddress(0,LINE3+id);
					glcdFont(0);

					if (id) {
						fprintf_P(lcd,PSTR("C:"));
					} else {                       fprintf_P(lcd,PSTR("A:"));}

					for (unsigned char i=0; i<5; i++) {

						if (submix[id].gain[i*2]||submix[id].gain[i*2+1]) {
							glcdFont(FP_NEGATIVE|((submix[id].invflags&(1<<(i*2)) ||submix[id].invflags&(1<<(i*2+1))) ? FP_ITALIC : 0));
						} else {
							glcdFont((submix[id].invflags&(1<<(i*2)) ||submix[id].invflags&(1<<(i*2+1))) ? FP_ITALIC : 0);
						}

						if (i<3) {
							if (i==2&&((mainmix[id].inpmix[i].flags&REZ_EXTCHAN))) {
								fprintf_P(lcd,PSTR(" 4"));
							} else {
								fprintf_P(lcd,PSTR(" %d"),i+1);
							}
						} else if (i==3) {
							fprintf_P(lcd,PSTR(" U"));							
						} else {
							fprintf_P(lcd,PSTR(" %c "),id?'D':'B');
						}
					}
					glcdFont(0);
					fprintf_P(lcd,PSTR(" "));
				}
				}
				
				glcdSetAddress(0,LINE8);
				fprintf_P(lcd,PSTR("Volume: -%.1d,%.1d dB   "),((1023-GetVolume())/8),(((1023-GetVolume())/4)%2) * 5);

				//nacteni stavu USB
				{

					unsigned char data[4]={0xEE,0,0,0};

					i2cMasterSend(TASM, 1, data);
					i2cMasterReceive(TASM, 4, data);

					glcdSetAddress(105,LINE4);
					glcdFont(0);
					if (!(data[3]&2)) {

						fprintf_P(lcd,PSTR("USB"));
					} else {

						fprintf_P(lcd,PSTR("     "));
					}

				}


				unsigned char data[10];
				signed short dbavg[10]={0,0,0,0,0,0,0,0,0,0};
				unsigned char i;

				for (unsigned char q=0; q<10; q++) {

					data[0]=0xFD;


					i2cMasterSend(TASM, 1, data);
					i2cMasterReceive(TASM, 10, data);

					for (unsigned char i=0; i<10; i++) {
						signed short db = 0;
						db = ((signed short)((unsigned char)((data[i]&0xF8)>>3))-19)*6;
						//			db = ((signed short)((unsigned char)((data[i]&0xF8)>>3))-15)*6;
						db += (data[i]&0x07)*6/8; //toto neni moc presne
						if (db>100) {db=100;}
						if (db<-100) {db=-100;}
						data[i]=db;

						dbavg2[i]+=db;
						if (db>dbmax[i]) {dbmax[i]=db; }

					}



					i = (q%2)*5+q/2;

					dbavg[i]=dbavg2[i]/10;
					dbavg2[i]=0;



					if (dbavg[i]>20) {dbavg[i]=20;}
					if (dbavg[i]<-20) {dbavg[i]=-20;}
					if (dbmax[i]>20) {dbmax[i]=20;}
					if (dbmax[i]<-17) {dbmax[i]=-17;}

					glcdSetDot(70,35);
					glcdSetDot(69,35);
					glcdSetDot(69,25);
					glcdSetDot(69,45);
					glcdSetDot(69,55);

					glcdSetDot(102,35);
					glcdSetDot(101,35);
					glcdSetDot(101,25);
					glcdSetDot(101,45);
					glcdSetDot(101,55);

					for (unsigned char p=0; p<2; p++) {
						glcdClrLine(71+i*3+p,15,71+i*3+p,55);
						glcdLine(71+i*3+p,(unsigned)((signed)35-dbavg[i]),71+i*3+p,55);
						if (p==1) {
							glcdSetDot(71+i*3+p,(unsigned)((signed)38-dbmax[i]));
						}
					}
					dbavg[i]=0;
					if (dbmax[i]>-120){dbmax[i]-=2;}
				}

			}

		};

		//VYPNUTO
		LCDBackOff();
		ResetLed(0x09);
		SetLed(0x08);


/*

   	glcdSetAddress(4,LINE2);
    fprintf_P(lcd,PSTR("Bubu's player"));





   glcdClearDisp();
   glcdRectangle(0,0,64,127);

   glcdProgress(8,57,118,70);
   glcdProgress(8,60,118,67);



   	//spektrak
   	for (unsigned char i=0;i<10;i++) {
   	 glcdSetDot(28+i*10,16);
   	 glcdSetDot(28+i*10,24);
   	 glcdSetDot(28+i*10,32);
   	 glcdSetDot(28+i*10,40);
   	}
   	glcdLine(28,47,126,47);

   	glcdFont(0);
   	glcdSetAddress(2,LINE2);
   	fprintf_P(lcd,PSTR("0"));
   	glcdSetAddress(2,LINE3);
   	fprintf_P(lcd,PSTR("-10"));
   	glcdSetAddress(2,LINE4);
   	fprintf_P(lcd,PSTR("-20"));
   	glcdSetAddress(2,LINE5);
   	fprintf_P(lcd,PSTR("-30"));
   	glcdSetAddress(2,LINE6);
   	fprintf_P(lcd,PSTR("-40"));

   	glcdLine(29,17,28+10,17);
   	for (unsigned char j=47;j>24;j--) {glcdLine(29,j,28+8,j);}
   	{
   	 unsigned char i=0;


   	 i++;
   	 glcdLine(29+i*10,20,28+10+i*10,20);
   	 for (unsigned char j=47;j>22;j--) {glcdLine(29+i*10,j,28+9+i*10,j);}
   	 i++;
   	 glcdLine(29+i*10,12,28+9+i*10,12);
   	 for (unsigned char j=47;j>15;j--) {glcdLine(29+i*10,j,28+9+i*10,j);}
   	 i++;
   	 glcdLine(29+i*10,14,28+9+i*10,14);
   	 for (unsigned char j=47;j>17;j--) {glcdLine(29+i*10,j,28+9+i*10,j);}
   	 i++;
   	 glcdLine(29+i*10,11,28+9+i*10,11);
   	 for (unsigned char j=47;j>9;j--) {glcdLine(29+i*10,j,28+9+i*10,j);}
   	 i++;
   	 glcdLine(29+i*10,15,28+9+i*10,15);
   	 for (unsigned char j=47;j>17;j--) {glcdLine(29+i*10,j,28+9+i*10,j);}
   	 i++;
   	 glcdLine(29+i*10,14,28+9+i*10,14);
   	 for (unsigned char j=47;j>19;j--) {glcdLine(29+i*10,j,28+9+i*10,j);}
   	 i++;
   	 glcdLine(29+i*10,12,28+9+i*10,12);
   	 for (unsigned char j=47;j>17;j--) {glcdLine(29+i*10,j,28+9+i*10,j);}
   	 i++;
   	 glcdLine(29+i*10,18,28+9+i*10,18);
   	 for (unsigned char j=47;j>21;j--) {glcdLine(29+i*10,j,28+9+i*10,j);}
   	 i++;
   	 glcdLine(29+i*10,17,28+8+i*10,17);
   	 for (unsigned char j=47;j>25;j--) {glcdLine(29+i*10,j,28+8+i*10,j);}
   	 i++;

   	}



   	//glcdRectangle(29,23,126,46);

   	glcdFont(FP_BOLD);
   	glcdSetAddress(10,LINE6);
    //fprintf_P(lcd,PSTR("HETLE - METLE"));
   	glcdFont(0);
   	glcdSetAddress(6,LINE7);
    fprintf_P(lcd,PSTR("EQ "));
   	glcdFont(FP_NEGATIVE);
    fprintf_P(lcd,PSTR(" LIM "));
   	glcdFont(0);
   	fprintf_P(lcd,PSTR(" "));
   	glcdFont(FP_NEGATIVE);
    fprintf_P(lcd,PSTR(" USB "));
   	glcdFont(0);
   	fprintf_P(lcd,PSTR("  Inp:"));
   	glcdFont(FP_ITALIC);
   	fprintf_P(lcd,PSTR("  CD"));
   	glcdFont(0);


   	glcdSetAddress(2,LINE8);
   	fprintf_P(lcd,PSTR("Vol:-48dB"));
   	glcdLine(0,10,0,62);
   	glcdLine(0,63,50,63);
   	glcdSetAddress(0,LINE1);
   	glcdFont(FP_NEGATIVE);
    fprintf_P(lcd,PSTR("      TLE-3    Preamp      "));



 */

		//USPORNY REZIM  --- VYPNUTO
		glcdClearDisp();
		glcdWriteBMP(15,LINE1,&Picture1);

		//ztlumeni
		for (unsigned char i=0; i<8; i++) {
			CSSetVolume(i,0xFF);
		}
		DelayMs(1000);
		while (GetButtons()){DelayMs(50);};
		glcdClearDisp();

		//vypnuti AP
		CodecResetOn(); APResetOn();

		//cekani na dalsi stisk PWR
		while (!(GetButtons()&BTNPWR)) {DelayMs(50);};
		while ((GetButtons()&BTNPWR)) {DelayMs(50);};

		//KONEC HLAVNI SMYCKY

		//reinit AP je v zacatku smycky
		redr=1;

	}
	while (1);




	return(0);
}


//*****************************************************************************
// Function: uart_putc
// Parameters: char to send
// Returns: 0
//
// Description: send a character to UART and return 0 for compatibility reasons
//*****************************************************************************
int uart_putc(char c)
{
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = c;
	return 0;
}


//*****************************************************************************
// Function: uart_getc
// Parameters: none
// Returns: the character read from UART
//
// Description: get a character from UART and return it, return -1 if the UART
//              was empty
//*****************************************************************************
int uart_getc(void)
{
	if (bit_is_set(UCSR0A, RXC) == 0)
		return -1;
	return UDR0;
}


//*****************************************************************************
// Function: mcuInit
// Parameters: none
// Returns: none
//
// Description: Initialize the ATMega128
//*****************************************************************************
void mcuInit(void)
{
	// Input/Output Ports initialization
	PORTA=0xFF; // Port A initialization
	DDRA=0xDF;

	PORTB=0x00; // Port B initialization
	DDRB=0x00;

	PORTC=0x00; // Port C initialization
	DDRC=0xFF;

	PORTD=0x00; // Port D initialization
	DDRD=0xFF;

	PORTE=0xF4; // Port E initialization
	DDRE=0x04;

	PORTF=0x00; // Port F initialization
	DDRF=0x00;

	PORTG=0xFF; // Port G initialization
	DDRG=0xFF;

	// Timer/Counter 0 initialization
	// Clock source: System Clock
	// Clock value: Timer 0 Stopped
	// Mode: Normal top=FFh
	// OC0 output: Disconnected
	ASSR=0x00;
	TCCR0=0x00;
	TCNT0=0x00;
	OCR0=0x00;

	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 16000,000 kHz
	// Mode: Normal top=FFFFh
	// OC1A output: Discon.
	// OC1B output: Discon.
	// OC1C output: Discon.
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	TCCR1A=0x00;
	TCCR1B=0x01;
	TCNT1H=0x00;
	TCNT1L=0x00;
	OCR1AH=0x00;
	OCR1AL=0x00;
	OCR1BH=0x00;
	OCR1BL=0x00;
	OCR1CH=0x00;
	OCR1CL=0x00;

	// Timer/Counter 2 initialization
	// Clock source: System Clock
	// Clock value: Timer 2 Stopped
	// Mode: Normal top=FFh
	// OC2 output: Disconnected
	TCCR2=0x00;
	TCNT2=0x00;
	OCR2=0x00;

	// Timer/Counter 3 initialization
	// Clock source: System Clock
	// Clock value: Timer 3 Stopped
	// Mode: Normal top=FFFFh
	// OC3A output: Discon.
	// OC3B output: Discon.
	// OC3C output: Discon.
	TCCR3A=0x00;
	TCCR3B=0x00;
	TCNT3H=0x00;
	TCNT3L=0x00;
	OCR3AH=0x00;
	OCR3AL=0x00;
	OCR3BH=0x00;
	OCR3BL=0x00;
	OCR3CH=0x00;
	OCR3CL=0x00;

	// External Interrupt(s) initialization
	// INT0: Off
	// INT1: Off
	// INT2: Off
	// INT3: Off
	// INT4: Off
	// INT5: Off
	// INT6: Off
	// INT7: Off
	EICRA=0x00;
	EICRB=0x00;
	EIMSK=0x00;

	// Timer(s)/Counter(s) Interrupt(s) initialization
	TIMSK=0x04;
	ETIMSK=0x00;

	// USART0 initialization
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART0 Receiver: Off
	// USART0 Transmitter: On
	// USART0 Mode: Asynchronous
	// USART0 Baud rate: 57600 (double speed = 115200)
	UCSR0A=0x02;
	UCSR0B=0x18;
	UCSR0C=0x06;
	UBRR0L=0x22;
	UBRR0H=0x00;

	// USART1 initialization
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART1 Receiver: Off
	// USART1 Transmitter: On
	// USART1 Mode: Asynchronous
	// USART1 Baud rate: 500000
	//... 0x00;
	UCSR1A=0x00;
	UCSR1B=0x08;
	UCSR1C=0x86;
	UBRR1L=0x01;
	//UBRR1L=0x10;
	UBRR1H=0x00;


	// Analog Comparator initialization
	// Analog Comparator: Off
	// Analog Comparator Input Capture by Timer/Counter 1: Off
	// Analog Comparator Output: Off
	ACSR=0x80;
	SFIOR=0x00;
}
